/**
 * Application bootstrap
 */
(function($) {
	'use strict';
	var wndw = $(window);

	function startApp() {
		// Bind events for touchable elements
		bindEvents();
		// Setup default ajax settings
		$.extend($.ajaxSettings,{
			timeout: 30000,
			contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			async: true,
			cache: false,
			headers: { "cache-control": "no-cache" },
			contents: {
				xml: /xml/,
				html: /html/,
				json: /json/
			},

			responseFields: {
				xml: "responseXML",
				text: "responseText",
				json: "responseJSON"
			},
			converters: {

				// Convert anything to text
				"* text": String,

				// Text to html (true = no transformation)
				"text html": true,

				// Evaluate text as a json expression
				"text json": jQuery.parseJSON,

				// Parse text as xml
				"text xml": jQuery.parseXML
			},

			// For options that shouldn't be deep extended:
			// you can add your own custom options here if
			// and when you create one that shouldn't be
			// deep extended (see ajaxExtend)
			flatOptions: {
				url: true,
				context: true
			}
		});
 
//		$.ajaxSetup({
//			timeout: 30000,  // 设置ajax超时时间为30秒
//			cache: false,
//			// http://stackoverflow.com/questions/12506897/is-safari-on-ios-6-caching-ajax-results
//			headers: { "cache-control": "no-cache" }
//		});
		// isTransitionAnimEnabled
		if( mobile_system === 'android' && device && parseInt(device.version) < 4 ) Views.isTransitionAnimEnabled = false;
		// Statistic
		if (Util.statisticEnabled) Util.statistic.onAppStart(VERSION_NUMBER);
		// Adjust layout size
		Layout.adjustWindowSize(wndw.width(), wndw.height());

		// Check version update
//		Util.checkUpdateOrNewInstall(function(result) {
//			if (result.isUpdate) {
//				// force user re-login
//				UserProfile.setUserLogout();
//			} else if (!result.isNewInstall && !result.isUpdate) {
//				Util.checkVersionUpdate();
//			}
//		});
		// Initialize app
		AppManager.initialize();
		// Initialize the File System
		FSManager.initialize();

		extendTemplateMethod();

		if (mobile_system && mobile_system === 'ios') {
			// 使用手动方式控制启动画面，只有在js啥的都初始化完成后才隐藏之
			// 参考：http://docs.phonegap.com/en/2.2.0/cordova_splashscreen_splashscreen.md.html#Splashscreen
			setTimeout(function() {
                       
				navigator.splashscreen.hide();
			}, 1000);
		}
	}

	window.networkState = 'No network connection';
	function checkConnection() {
		var networkState = navigator.network.connection.type;
		var states = {};
		states[Connection.UNKNOWN] = 'Unknown connection';
		states[Connection.ETHERNET] = 'Ethernet connection';
		states[Connection.WIFI] = 'WiFi connection';
		states[Connection.CELL_2G] = 'Cell 2G connection';
		states[Connection.CELL_3G] = 'Cell 3G connection';
		states[Connection.CELL_4G] = 'Cell 4G connection';
		states[Connection.NONE] = 'No network connection';
		window.networkState = states[networkState];
		Util.networkStateChanged(window.networkState);
		// 如果开启了移动统计...
		if (Util.statisticEnabled) Util.statistic.onConnectionChanged();
	}

	var device_ready = false;
	var document_ready = false;
	var template_ready = false;
	function initApp() {
 
		if ((device_ready && document_ready && template_ready) ||
			(document_ready && template_ready && !mobile_system)) {
			startApp();
		}
	}
	$(function() {
		Util.log('application start!');
		document_ready = true;
		if ($('#J_templateContainer').length > 0) {
			Util.log('从页面编译模板...');
			$.each(Views.tmpl, function(index, value) {
				Views.tmpl[index] = template.compile($('#template_' + index).html());
			});
			template_ready = true;
			$('#J_templateContainer').remove();
		} else {
			// 注意：使用这种方式，需要给Chrome浏览器加上--allow-file-access-from-files启动参数
			Util.log('从文件编译模板...');
			var _tmplArr = [];
			$.each(Views.tmpl, function(key, value) {
				_tmplArr.push([key, value]);
			});
			(function loadTmplComplie(tmplUrl) {
				if (!tmplUrl) {
					template_ready = true;
					initApp();
				} else {
					$.ajax({
						url: tmplUrl[1],
						type: 'POST',
						data: {
							rtime: Util.rtime()
						},
						dataType: 'text',
						success: function(data) {
							Views.tmpl[tmplUrl[0]] = template.compile(data);
							loadTmplComplie(_tmplArr.pop());
						},
						error: function() {
							Util.log('未找到模板文件' + tmplUrl[1]);
						}
					});
				}
			}) (_tmplArr.pop());
		}
		FastClick.attach(document.body);
		initApp();
	});
	document.addEventListener('deviceready', function() {
		device_ready = true;
		document.addEventListener('online', checkConnection, false);
		document.addEventListener('offline', checkConnection, false);
		if  ( 'StatusBar' in window ) {
			StatusBar.overlaysWebView( false );
            StatusBar.show();
		}
		checkConnection();
		initApp();
	}, false);
	document.addEventListener('resume', function() {
	}, false);

	function extendTemplateMethod(){
		template.helper('assistResTime', function(){
			return '?t='+new Date().getTime();
		});
		template.helper('getSpaceTime', function(spaceTime){
			return DateUtil.getSpaceTime(spaceTime);
		});
		template.helper('getFutureSpaceTime', function(spaceTime){
			return DateUtil.getFutureSpaceTime(spaceTime);
		});
		template.helper('getFutureSpaceTime2', function(spaceTime){
			return DateUtil.getFutureSpaceTime2(spaceTime);
		});
		template.helper('ceil', function (number) {
			return Math.ceil(Number(number));
		});
		template.helper('replaceBr', function (content) {
			return (content.replaceAll('<br/>', '')).replaceAll('<br>', '');
		});
		template.helper('getStaticStr', function (key) {
			if(key == 'mobile_system'){
				return mobile_system;
			}
		});
	}

	function bindEvents() {
		var hasTouch = 'ontouchstart' in window,
			RESIZE_EVENT = 'onorientationchange' in window ? 'orientationchange' : 'resize',
			TOUCH_EVENT = hasTouch ? 'touchstart touchend touchcancel touchmove' : 'mousedown mouseup mouseout mousemove',
			TOUCHDOWN_TARGET, TOUCH_LONGTAP_TID = null;
		$(document.body).on(TOUCH_EVENT, '.ui-btn', function(e) {
			var _eType = e.type,
				_btn = $(this),
				_btnId = _btn.attr('id');
			if (!_btnId) {
				_btnId = Util.getAutoID();
				_btn.attr('id', _btnId);
			}
			if (hasTouch && e.originalEvent.touches.length > 1) {
				Util.log('Multi-touch is not supported.');
				if (TOUCHDOWN_TARGET) {
					$('#' + TOUCHDOWN_TARGET).removeClass('down');
					TOUCHDOWN_TARGET = null;
				}
				if (TOUCH_LONGTAP_TID) {
					clearTimeout(TOUCH_LONGTAP_TID);
					TOUCH_LONGTAP_TID = null;
				}
				return;
			}
			if (_eType === 'touchstart' || _eType === 'mousedown') {
				TOUCHDOWN_TARGET = _btnId;
				if (!_btn.hasClass('down')) _btn.addClass('down');
				if (_btn.attr('data-action')) {
					e.stopPropagation();
				} else if (_btn.attr('data-downAction')) {
					e.preventDefault();
					setTimeout(function() {
						if (!Views.isAnyViewInTransition) Callback[_btn.attr('data-downAction')](_btnId);
					}, 0);
					if (!_btn.attr('data-upAction')) {
						_btn.removeClass('down');
						TOUCHDOWN_TARGET = null;
					}
				} else if (_btn.attr('data-longTapAction')) {
					e.preventDefault();
					if (TOUCH_LONGTAP_TID) clearTimeout(TOUCH_LONGTAP_TID);
					TOUCH_LONGTAP_TID = setTimeout((function(btnId) {
						return function() {
							TOUCHDOWN_TARGET = null;
							TOUCH_LONGTAP_TID = null;
							Callback[$('#' + btnId).removeClass('down').attr('data-longTapAction')](btnId);
						};
					})(_btnId), 2000);
				}
			} else if (((_eType === 'touchmove' || _eType === 'mousemove') && Views.isAnyViewScrolling) || Util.inMotionTrack || _eType === 'touchcancel' || _eType === 'mouseout') {
				_btn.removeClass('down');
				TOUCHDOWN_TARGET = null;
				if (_btn.attr('data-upAction')) {
					e.preventDefault();
					setTimeout(function() {
						if (!Views.isAnyViewInTransition) Callback[_btn.attr('data-upAction')](_btnId);
					}, 0);
				}
				if (TOUCH_LONGTAP_TID) {
					clearTimeout(TOUCH_LONGTAP_TID);
					TOUCH_LONGTAP_TID = null;
				}
			} else if (_eType === 'touchend' || _eType === 'mouseup') {
				if (TOUCHDOWN_TARGET === _btnId) {
					_btn.removeClass('down');
					TOUCHDOWN_TARGET = null;
					if (_btn.attr('data-action')) {
						e.preventDefault();
						setTimeout(function() {
							if (!Views.isAnyViewInTransition) Callback[_btn.attr('data-action')](_btnId);
						}, 0);
						_btn.addClass('active');
					} else if (_btn.attr('data-upAction')) {
						e.preventDefault();
						setTimeout(function() {
							if (!Views.isAnyViewInTransition) Callback[_btn.attr('data-upAction')](_btnId);
						}, 0);
					}
					if (TOUCH_LONGTAP_TID) {
						clearTimeout(TOUCH_LONGTAP_TID);
						TOUCH_LONGTAP_TID = null;
					}
				}
			}
		});

		document.addEventListener('touchmove', function(e) {
			e.preventDefault();
		}, false);
		// for Android only
		document.addEventListener('backbutton', function(e) {
            if (!Views.isAnyViewInTransition) {
                Callback.pageBack();
            }

		}, false);
		wndw.on(mobile_system === 'android' ? 'resize' : RESIZE_EVENT, function() {
			Layout.adjustWindowSize(wndw.width(), wndw.height());
		});
	}
}) (jQuery);

