(function($) {
	'use strict';

	if (typeof window.device === 'undefined') window.device = { uuid: 'unknown' };
	
	Util.statisticEnabled = (APPVERSION !== 'web' && !Util.data('statistic.disabled'));

	var EVENTS_COUNT_THRESHOLD = 10, // 收集到10条就发送
		EVENTS_COUNT_LIMIT = 500, // 如果一直发送失败，本地数据允许保存的记录量
		EVENTS_SEND_INTERVAL = 3600000, // 两次发送间隔1个小时也发送一下
		sendOnlyInWifi = !!Util.data('statistic.sendOnlyInWifi');

	function saveEvent(data) {
		var events = Util.data('statistic.events');
		if (!events) events = [];
		if (events.length >= EVENTS_COUNT_LIMIT)
			events.shift();
		events.push(data);
		Util.data('statistic.events', events);
		// 首次安装启动或者更新后启动，直接发送统计数据
		if (data.event === 'first' || data.event === 'upgrade') {
			sendStatistic();
		} else {
			// 检查是否要发送数据
			var lastSendMillis = Util.data('statistic.lastSendMillis') || Date.now();
			if (events.length == EVENTS_COUNT_THRESHOLD ||
				Date.now() - lastSendMillis >= EVENTS_SEND_INTERVAL ||
				(events.length > EVENTS_COUNT_THRESHOLD && Util.isOnline() /* 之前发送失败可能是无网络连接 */)) {
				if (sendOnlyInWifi && typeof navigator.connection !== 'undefined' && navigator.connection.type !== Connection.WIFI)
					return; // 只在有WIFI的情况下发送
				sendStatistic();
			}
		}
	}

	function sendStatistic() {
		var events = Util.data('statistic.events');
		if (!events || !events.length) return;

		Util.data('statistic.events', []);
		Util.data('statistic.lastSendMillis', Date.now());
		var stat = {
			device_uuid: device.uuid,
			app_id: APP_ID,
			events: events
		};
		Util.log('Sending statistic at ' + new Date());
		$.ajax({
			type : 'POST',
			url  : STATISTIC_URL,
			data : { stat: JSON.stringify(stat) },
			dataType: 'json'
		}).fail(function() {
			Util.log('Sending statistic failed...');
			// 发送失败，保留events
			var newEvents = Util.data('statistic.events');
			Util.data('statistic.events', events.concat(newEvents));
		});
	}

	Util.statistic = {
		setSendOnlyInWifi: function(b) {
			sendOnlyInWifi = !!b;
			Util.data('statistic.sendOnlyInWifi', sendOnlyInWifi);
		},
	
		onAppStart: function(currVersion) {
			var event = '',
				lastVersion = Util.data('statistic.version');
			if (!lastVersion) {
				Util.data('statistic.version', currVersion);
				event = 'first';
			} else if (lastVersion !== currVersion) {
				Util.data('statistic.version', currVersion);
				event = 'upgrade';
			} else {
				saveEvent({
					event: 'startup',
					timestamp: Date.now(),
					channel_id: CHANNEL_ID
				});
				return;
			}
			
			var data = {
				event: event,
				timestamp: Date.now(),
				channel_id: CHANNEL_ID,
				device_model: device.model,
				device_platform: device.platform,
				device_version: device.version,
				app_version: currVersion
			};
			if (event === 'upgrade') {
				data.app_prev_version = lastVersion;
			}
			saveEvent(data);
		},

		onAppExit: function() {
			saveEvent({
				event: 'close',
				timestamp: Date.now(),
				channel_id: CHANNEL_ID
			});
		},

		onConnectionChanged: function() {
			var event = 'online', network = 'unknown';
			if (typeof navigator.connection !== 'undefined') {
				switch (navigator.connection.type) {
				case Connection.ETHERNET:
					network = 'ethernet';
					break;
				case Connection.WIFI:
					network = 'wifi';
					break;
				case Connection.CELL_2G:
					network = '2g';
					break;
				case Connection.CELL_3G:
					network = '3g';
					break;
				case Connection.CELL_4G:
					network = '4g';
					break;
				case Connection.CELL:
					network = 'cell';
					break;
				case Connection.NONE:
					network = 'none';
					event = 'offline';
					break;
				default:
					break;
				}
			}
			saveEvent({
				event: event,
				timestamp: Date.now(),
				channel_id: CHANNEL_ID,
				network: network
			});
		},

		onPageOpen: function(pageId, pageName) {
			saveEvent({
				event: 'open',
				timestamp: Date.now(),
				channel_id: CHANNEL_ID,
				page_id: pageId,
				page_name: pageName
			});
		}
	};

	$.each(['pause', 'resume'], function(_, event) {
		document.addEventListener(event, function() {
			saveEvent({
				event: event,
				timestamp: Date.now(),
				channel_id: CHANNEL_ID
			});
		}, false);
	});
}) (jQuery);
