// ------------------------------------------------------------------------
// OO Class
//
var Class = {
	create: function() {
		return function() {
			this.initialize.apply(this, arguments);
		};
	},

	extend: function() {
		var child = arguments[0],
			parents = [].splice.call(arguments, 1);
		if (!($.isFunction(child)))
			throw 'The first parameter must be a first-level function.';
		if (!parents.length)
			throw 'You must provide at least a parent class to extend';
		for (var i = 0; i < parents.length; ++i) {
			var p = parents[i];
			$.extend(child.prototype, $.isFunction(p) ? p.prototype : p);
		}
		child.__super__ = function(parentClass) {
			if (!parentClass)
				parentClass = parents[0];
			return $.isFunction(parentClass) ? parentClass.prototype : parentClass;
		};
		return child;
	}
};


// ------------------------------------------------------------------------
// Animation manager
//
var FX = (function($, doc) {
	'use strict';
	var dummyStyle = doc.createElement('div').style,
		vendor = (function() {
			var vendors = 't,webkitT,MozT,msT,OT'.split(','),
				t,
				i = 0,
				l = vendors.length;
			for (; i < l; i++) {
				t = vendors[i] + 'ransform';
				if (t in dummyStyle) {
					return vendors[i].substr(0, vendors[i].length - 1);
				}
			}
			return false;
		})(),
		animEndEventName = 'webkitAnimationEnd';/*(function () {
			if (vendor === false) return false;
			var animationEnd = {
				''			: 'animationend',
				'webkit'	: 'webkitAnimationEnd',
				'Moz'		: 'animationend',
				'O'			: 'oanimationend',
				'ms'		: 'MSAnimationEnd'
			};
			return animationEnd[vendor];
		})();*/

	return {
		animate: function(jqEl, animClass) {
			var deferred = $.Deferred();
			if (Views.isTransitionAnimEnabled) {
				jqEl.addClass(animClass).on(animEndEventName, function() {
					jqEl.removeClass(animClass).off(animEndEventName);
					deferred.resolve(jqEl);
				});
			} else {
				setTimeout(function() {
					deferred.resolve(jqEl);
				}, 0);
			}
			return deferred.promise();
		},

		concurrent: function() {
			return $.when.apply($, arguments);
		}
	};
}) (jQuery, document);

/* AnimManager provides just language sugar */
var AnimManager = {
	one: function(jqEl, animClass, callback) {
		return FX.animate(jqEl, animClass).done(callback || $.noop);
	},

	two: function(jqEl1, animClass1, jqEl2, animClass2, callback) {
		return FX.concurrent(
				FX.animate(jqEl1, animClass1), FX.animate(jqEl2, animClass2)
			).done(callback || $.noop);
	},

	three: function(jqEl1, animClass1, jqEl2, animClass2, jqEl3, animClass3, callback) {
		return FX.concurrent(
				FX.animate(jqEl1, animClass1), FX.animate(jqEl2, animClass2), FX.animate(jqEl3, animClass3)
			).done(callback || $.noop);
	}
};


// ------------------------------------------------------------------------
// FileSystem manager
//
var FSManager = {
	initialize: function() {
		if (!mobile_system || this.rootPath)
			return;
		var self = this;
		window.requestFileSystem(LocalFileSystem.PERSISTENT, 0,
			function(fs) {
				self.fs = fs;
				fs.root.getDirectory('data-citizen', { create: true, exclusive: false },
					function(dir) {
						self.rootPath = dir.toURL();
						if (self.rootPath.lastIndexOf('/') === self.rootPath.length - 1)
							self.rootPath = self.rootPath.substr(0, self.rootPath.length - 1);
						Util.log('rootPath: ' + self.rootPath);
					},
					function(err) {
						Util.log('Failed to getDirectory: ' + err.code);
					}
				);
				// Continue upload or download
				self.uploadQueue = Util.data('FSManager.uploadQueue') || [];
				if (self.uploadQueue.length > 0) {
					Util.log('There are pending uploads, continue >>');
					Util.log(self.uploadQueue);
					self._startUpload();
				}
				self.downloadQueue = Util.data('FSManager.downloadQueue') || [];
				if (self.downloadQueue.length > 0) {
					Util.log('There are pending downloads, continue >>');
					Util.log(self.downloadQueue);
					self._startDownload();
				}
			},
			function(evt) {
				Util.log('Could not get LocalFileSystem: ' + evt.target.error.code);
			}
		);
	},

	getRootPath: function() {
		return this.rootPath;
	},

	_getRootPathDir: function() {
		if (!this.rootPathDir)
			this.rootPathDir = new DirectoryEntry('data-mcs', this.rootPath);
		return this.rootPathDir;
	},

	/**
	 * The given options is very like the one provided to jQuery.ajax, which has the folliwng fields:
	 * {
	 *    file        URL of the full path to the upload file (The <code>fileKey</code> will always be 'file')
	 *    url         The Server url
	 *    mimeType    (Optional) The MIME type of the file, such as "text/plain", "image/jpeg", "image/png", etc, defaulting to "image/jpeg".
	 *    data        (Optional) Parameters to be sent together to the server
	 *
	 * !! The following callbacks must register them on the Callback scope, and provide the function name in string as the parameter.
	 *
	 *    onStart     (Optional) Callback when the upload starts, passed in with a FileTransfer object.
	 *    onProgress  (Optional) Called with a ProgressEvent whenever a new chunk of data is transferred,
	 *                with the following fields:
	 *                  - file          The file to be uploaded
	 *                  - lengthComputable  On both Android an iOS, lengthComputable is false for downloads that use gzip encoding
	 *                  - loaded
	 *                  - total
	 *    onSuccess   (Optional) Callback when the upload completes successfully
	 *                This callback is called with a FileUploadResult object of the following fields:
	 *                  - bytesSent     The number of bytes sent to the server as part of the upload. (long)
	 *                  - responseCode  The HTTP response code returned by the server. (long)
	 *                  - response      The HTTP response returned by the server. (DOMString)
	 *    onError     (Optional) Callback when the upload fails
	 *                This callback is called with a FileTransferError object of the following fields:
	 *                  - code          One of the predefined error codes listed below. (Number)
	 *                  - source        URI to the source (String)
	 *                  - target        URI to the target (String)
	 *                  - http_status   HTTP status code. This attribute is only available when a response code is received from the HTTP connection. (Number)
	 * }
	 *
	 * NOTE: The upload does *NOT* occur immediately. If one upload is progressing, other uploads added
	 *       later will be queued.  The <code>onStart</code> callback is used to acknowledge when the
	 *       upload starts.
	 */
	upload: function(options, priorityHigh) {
		// validate
		if (!options.file || !options.url) {
			console.error('file / url is required field.');
			return;
		}
		if (this._isFileInUploadQueue(options, priorityHigh)) {
			console.log('Already in upload queue: ' + options.file);
			return;
		}
		if (!this._validateCallbacks(options)) {
			console.error('Callback functions must be registered to Callback scope and provided as string.');
			return;
		}
		if (priorityHigh && this.uploadQueue.length > 1)
			this.uploadQueue.insert(1, options);
		else
			this.uploadQueue.push(options);
		Util.data('FSManager.uploadQueue', this.uploadQueue);

		if (this.uploadQueue.length === 1) this._startUpload();
	},

	/**
	 * The given options is very like the one provided to jQuery.ajax, which has the folliwng fields:
	 * {
	 *    url         The Server url of the file to be downloaded
	 *    file        URL of the full to store the downloaded file
	 *
	 * !! The following callbacks must register them on the Callback scope, and provide the function name in string as the parameter.
	 *
	 *    onStart     (Optional) Callback when the download starts, passed in with a FileTransfer object.
	 *    onProgress  (Optional) Called with a ProgressEvent whenever a new chunk of data is transferred,
	 *                with the following fields:
	 *                  - file          The file to be downloaded
	 *                  - lengthComputable  On both Android an iOS, lengthComputable is false for downloads that use gzip encoding
	 *                  - loaded
	 *                  - total
	 *    onSuccess   (Optional) Callback when the download completes successfully, which is called with an FileEntry object.
	 *    onError     (Optional) Callback when the download fails
	 *                This callback is called with a FileTransferError object of the following fields:
	 *                  - code          One of the predefined error codes listed below. (Number)
	 *                  - source        URI to the source (String)
	 *                  - target        URI to the target (String)
	 *                  - http_status   HTTP status code. This attribute is only available when a response code is received from the HTTP connection. (Number)
	 * }
	 *
	 * NOTE: The download does *NOT* occur immediately. If one download is progressing, other uploads added
	 *       later will be queued.  The <code>onStart</code> callback is used to acknowledge when the
	 *       download starts.
	 */
	download: function(options, priorityHigh) {
		// validate
		if (!options.url || !options.file) {
			console.error('url / file is required field.');
			return;
		}
		if (this._isFileInDownloadQueue(options, priorityHigh)) {
			console.log('Already in download queue: ' + options.file);
			return;
		}
		if (!this._validateCallbacks(options)) {
			console.error('Callback functions must be registered to Callback scope and provided as string.');
			return;
		}
		if (priorityHigh && this.downloadQueue.length > 1)
			this.downloadQueue.insert(1, options);
		else
			this.downloadQueue.push(options);
		Util.data('FSManager.downloadQueue', this.downloadQueue);

		if (this.downloadQueue.length === 1) this._startDownload();
	},

	/**
	 * Move a file to a different location on the file system
	 *
	 * src - The file to be moved.
	 * newName - The new name of the file. Defaults to the current name if unspecified.
	 * successCallback - A callback that is called with the FileEntry object of the new file.
	 * errorCallback - A callback that is called if an error occurs when attempting to move the file. Invoked with a FileError object.
	 */
	move: function(src, newName, successCallback, errorCallback) {
		var dir = this._getRootPathDir(),
			isURL = src.startsWith('file://localhost');
		if (isURL) {
			window.resolveLocalFileSystemURI(src,
				function(f) {
					Util.log('URL resolved: ' + src + ' --> ' + f.toURL());
					f.moveTo(dir, newName, successCallback, errorCallback);
				},
				function(evt) {
					errorCallback(evt.target.error);
				});
		} else {
			this.fs.root.getFile(src, { create: false, exclusive: false },
				function(f) {
					f.moveTo(dir, newName, successCallback, errorCallback);
				}, errorCallback);
		}
	},

	/**
	 * Copy a file to a new location on the file system
	 *
	 * src - The file to be copied.
	 * newName - The new name of the file. Defaults to the current name if unspecified.
	 * successCallback - A callback that is called with the FileEntry object of the new file.
	 * errorCallback - A callback that is called if an error occurs when attempting to move the file. Invoked with a FileError object.
	 */
	copy: function(src, newName, successCallback, errorCallback) {
		var dir = this._getRootPathDir(),
			isURL = src.startsWith('file://localhost');
		if (isURL) {
			window.resolveLocalFileSystemURI(src,
				function(f) {
					Util.log('URL resolved: ' + src + ' --> ' + f.toURL());
					f.copyTo(dir, newName, successCallback, errorCallback);
				},
				function(evt) {
					errorCallback(evt.target.error);
				});
		} else {
			this.fs.root.getFile(src, { create: false, exclusive: false },
				function(f) {
					f.copyTo(dir, newName, successCallback, errorCallback);
				}, errorCallback);
		}
	},

	/**
	 * Checks if the specified file existed on the file syste.
	 */
	isFileExisted: function(file, existCallback) {
		if (!this.fs) return;
		this.fs.root.getFile(file, { create: false, exclusive: false },
			function(f) {
				existCallback(true);
			},
			function(e) {
				existCallback(false);
			}
		);
	},

	/**
	 * Delete the specified file from the file system.
	 */
	remove: function(file, completeCallback) {
		this.fs.root.getFile(file, { create: false, exclusive: false },
			function(f) {
				f.remove();
				completeCallback();
			},
			function(ignore) {
				completeCallback();
			}
		);
	},

	computeDataFolderSize: function(computedCallback, errorCallback) {
		this.fs.root.getDirectory('data-mcs', { create: false, exclusive: false },
			function(dir) {
				dir.createReader().readEntries(function(entries) {
					var totalSize = 0, count = entries.length, computed = 0;
					if (count === 0) {
						computedCallback(totalSize);
						return;
					}
					function _success(f) {
						//Util.log('File: ' + f.toURL() + ' --> Size: ' + f.size);
						totalSize += f.size;
						if (++computed === count) computedCallback(totalSize);
					}
					function _error(e, err) {
						Util.log('Could not access: ' + e.toURL() + ': ' + err.code);
						if (++computed === count) computedCallback(totalSize);
					}
					for (var i = 0; i < count; ++i) {
						var e = entries[i];
						if (e.isDirectory) {
							Util.log('WARNING: This directory should not exist: ' + e.toURL());
							if (++computed === count) computedCallback(totalSize);
							continue;
						}
						e.file(_success, _error.bind(e));
					}
				}, function(err) {
					Util.log('Failed to readEntries under ' + dir.toURL() + ': ' + err.code);
					errorCallback(err);
				});
			},
			function(err) {
				Util.log('Failed to getDirectory: ' + err.code);
				errorCallback(err);
			}
		);
	},

	clearDataFolder: function(successCallback, errorCallback) {
		this.fs.root.getDirectory('data-mcs', { create: false, exclusive: false },
			function(dir) {
				dir.createReader().readEntries(function(entries) {
					var count = entries.length, removed = 0;
					if (count === 0) {
						successCallback(count);
						return;
					}
					function _success() {
						if (++removed === count) successCallback(count);
					}
					function _error(e, err) {
						Util.log('Could not access: ' + e.toURL() + ': ' + err.code);
						if (++removed === count) successCallback(count);
					}
					for (var i = 0, len = entries.length; i < len; ++i) {
						var e = entries[i];
						//Util.log('Removing: ' + e.toURL());
						if (e.isDirectory)
							e.removeRecursively(_success, _error.bind(e));
						else
							e.remove(_success, _error.bind(e));
					}
				}, function(err) {
					Util.log('Failed to readEntries under ' + dir.toURL() + ': ' + err.code);
					errorCallback(err);
				});
			},
			function(err) {
				Util.log('Failed to getDirectory: ' + err.code);
				errorCallback(err);
			}
		);
	},

	debugUploadOrDownloadError: function(err) {
		Util.log('code:' + err.code + ' source:' + err.source + ' target:' + err.target + ' http_status:' + err.http_status);
	},

	_validateCallbacks: function(opt) {
		var fields = ['onStart', 'onProgress', 'onSuccess', 'onError'];
		for (var i = 0; i < fields.length; ++i) {
			var callback = opt[fields[i]];
			if (callback && typeof callback !== 'string')
				return false;
		}
		return true;
	},

	// ----- Upload utilities

	_isFileInUploadQueue: function(options, priorityHigh) {
		for (var i = 0; i < this.uploadQueue.length; ++i) {
			var opt = this.uploadQueue[i];
			if (opt.url === options.url && opt.file === options.file) {
				if (i > 1 && priorityHigh)
					this.uploadQueue.insert(1, this.uploadQueue.remove(i));
				return true;
			}
		}
		return false;
	},

	_startUpload: function() {
		if (!this.uploadQueue.length)
			return;
		var opt = this.uploadQueue[0];

		var uploadOptions = new FileUploadOptions();
		uploadOptions.fileKey  = 'file';
		uploadOptions.fileName = opt.file.substr(opt.file.lastIndexOf('/')+1);
		uploadOptions.mimeType = opt.mimeType || 'image/jpeg';
		uploadOptions.params   = opt.data || {};

		var ft = new FileTransfer();
		if (opt.onProgress) ft.onprogress = this._onprogressWrapper.bind(this, opt);
		ft.upload(opt.file, encodeURI(opt.url), this._uploadSuccess.bind(this, opt), this._uploadError.bind(this, opt), uploadOptions);
		if (opt.onStart)
			try { Callback[opt.onStart](ft); } catch (e) {}
		else
			Util.log('Start to upload: ' + opt.file + ' --> ' + opt.url);
	},

	_nextUpload: function() {
		this.uploadQueue.shift();
		Util.data('FSManager.uploadQueue', this.uploadQueue);
		this._startUpload();
	},

	_onprogressWrapper: function(opt, progressEvt) {
		progressEvt.file = opt.file;
		try {
			Callback[opt.onProgress](progressEvt);
		} catch (e) {}
	},

	_uploadSuccess: function(opt, r) {
		if (opt.onSuccess) try { Callback[opt.onSuccess](r); } catch (e) {}
		this._nextUpload();
	},

	_uploadError: function(opt, error) {
		if (opt.onError) try { Callback[opt.onError](error); } catch (e) {}
		this._nextUpload();
	},

	// ----- Download utilities

	_isFileInDownloadQueue: function(options, priorityHigh) {
		for (var i = 0; i < this.downloadQueue.length; ++i) {
			var opt = this.downloadQueue[i];
			if (opt.url === options.url && opt.file === options.file) {
				if (i > 1 && priorityHigh)
					this.downloadQueue.insert(1, this.downloadQueue.remove(i));
				return true;
			}
		}
		return false;
	},

	_startDownload: function() {
		if (!this.downloadQueue.length)
			return;
		var opt = this.downloadQueue[0];
		var ft = new FileTransfer();
		if (opt.onProgress) ft.onprogress = this._onprogressWrapper.bind(this, opt);
		ft.download(encodeURI(opt.url), opt.file, this._downloadSuccess.bind(this, opt), this._downloadError.bind(this, opt));
		if (opt.onStart)
			try { Callback[opt.onStart](ft); } catch (e) {}
		else
			Util.log('Start to download: ' + opt.url + ' --> ' + opt.file);
	},

	_nextDownload: function() {
		this.downloadQueue.shift();
		Util.data('FSManager.downloadQueue', this.downloadQueue);
		this._startDownload();
	},

	_downloadSuccess: function(opt, fileEntry) {
		if (opt.onSuccess) try { Callback[opt.onSuccess](fileEntry, opt.url); } catch (e) {}
		this._nextDownload();
	},

	_downloadError: function(opt, error) {
		if (opt.onError) try { Callback[opt.onError](error); } catch (e) {}
		this._nextDownload();
	}
};
Callback.uploadOrDlStart = function(ft) {
	Util.log('start: ' + ft.url);
};
Callback.uploadOrDlProgress = function(evt) {
	Util.log('file:' + evt.file + '  lengthComputable:' + evt.lengthComputable + ' percent:' + Math.round(evt.loaded / evt.total * 100) + '%');
};
Callback.uploadSuccess = function(r) {
	Util.log('bytesSent:' + r.bytesSent + ' responseCode:' + r.responseCode + ' response:' + r.response);
};
Callback.downloadSuccess = function(f, url) {
	Util.log('File downloaded: ' + url + ' --> ' + f.toURL());
};
Callback.uploadOrDlError = function(err) {
	Util.log('code:' + err.code + ' source:' + err.source + ' target:' + err.target + ' http_status:' + err.http_status);
};



// ------------------------------------------------------------------------
// Layout foundation
// 
var Layout = {
	windowWidth  : 0,
	windowHeight : 0,
	statusBarMargin: 0,
	adjustWindowSize: function(w, h) {
		Util.log('Window size adjusted, width: ' + w + ', height: ' + h);
		this.windowWidth  = w;
		this.windowHeight = h;
		if (mobile_system === 'ios' && parseInt(device.version, 10) >= 7)
			this.statusBarMargin = 20;
		$('#J_cssAppend').html(Views.tmpl.cssAppend({
			windowWidth  : w,
			windowHeight : h,
			statusBarMargin: this.statusBarMargin
		}));
		if (StageManager.getCurrentView()) {
			StageManager.getCurrentView().onResize(w, h);
		}
	}
};

Layout.LayoutBase = {
	willShow: function(ct) {
		// Add the layout to the container
	},

	didShow: function(ct) {
		// Do additional stuff after the Stage view utilizing this layout has been shown
	},

	willHide: function() {
		// This view is about to hide
	},

	didHide: function() {
		// This view is hidden (NOTE: The DOM will be destroyed after this method call)
	},

	onResize: function(w, h) {
		// Need to adjust the size 
	}
};


// ------------------------------------------------------------------------
// Widget foundation
// 
var Widget = {
	popupWrapperHideDelay: null,
	popupWrapper: function() {
		if (this.popupWrapperHideDelay) {
			clearTimeout(this.popupWrapperHideDelay);
			this.popupWrapperHideDelay = null;
		}
		var w = $('#popupWrapper');
		if (w.length === 0) {
			w = $('<div id="popupWrapper" class="ui-overlay-fullscreen"></div>').appendTo($('body'));
		}
		return w;
	},
	popupWrapperHide: function(delay, callback) {
		$('#popupWrapper').html('');
		var delayMillis = (typeof delay === 'number') ? delay : 0;
		if (!delayMillis) {
			$('#popupWrapper').hide();
			if (callback) callback();
		} else {
			this.popupWrapperHideDelay = setTimeout(function() {
				$('#popupWrapper').hide();
				if (callback) callback();
			}, delayMillis);
		}
	},
	popupIsDisplayed: function() {
		var w = $('#popupWrapper');
		return w.length === 1 && w.css('display') === 'block';
	},
	popupPositionCenter: function() {
		var _popup = $('#popupWrapper>div'),
			_height = _popup.outerHeight();
		_popup.css({
			'margin-top': '-' + _height / 2 + 'px'
		});
	},
	popupSuccess: function(message, callback, ellapse) {
		var w = this.popupWrapper();
		w.show().html(Views.tmpl.popupSuccess({
			title: message ? message : '提示消息内容'
		}));
		this.popupPositionCenter();
		if (callback) callback();
		ellapse = ellapse || 1200;
		this.popupWrapperHideDelay = setTimeout(function() {
			$('#popupWrapper').html('').hide();
		}, ellapse);
	},
	popupFailure: function(message, callback, ellapse) {
		var w = this.popupWrapper();
		w.show().html(Views.tmpl.popupFailure({
			title: message ? message : '提示消息内容'
		}));
		this.popupPositionCenter();
		if (callback) callback();
		ellapse = ellapse || 1200;
		this.popupWrapperHideDelay = setTimeout(function() {
			$('#popupWrapper').html('').hide();
		}, ellapse);
	},
	popupLoading: function(message) {
		var w = this.popupWrapper();
		w.show().html(Views.tmpl.popupLoading({
			title: message ? message : '加载中，请稍候...'
		}));
		this.popupPositionCenter();
	},
	popupLoadingChangeTitle: function(title) {
		var w = this.popupWrapper();
		w.find('ui-popup-title').html(title);
	},
	popupLoadingClose: function() {
		this.popupWrapperHide();
	},
	popupWithCustomView: function(view) {
		if (!this.__popupWrapperIdx) {
			this.__popupWrapperIdx = 1;
			this.__popupWrapperCustomViewMap = {};
		} else {
			this.__popupWrapperIdx++;
		}
		var id = 'popupWrapper' + this.__popupWrapperIdx,
			w = $('<div id="' + id + '" class="ui-overlay-fullscreen"></div>').appendTo($('body'));
		view.__popupWrapperId = id;  // memorize
		this.__popupWrapperCustomViewMap[id] = view;
		w.html(Views.tmpl.popupCustomView($.extend({
				id: id,
				width: 300,
				title: '温馨提示',
				btnLeftLabel: '取消',
				btnLeftAction: 'popupCancel',
				btnRightLabel: '确定',
				btnRightAction: 'popupOk'
			}, view.options || {})));
		var dlg = w.find('div.ui-popup-custom'),
			ct = w.find('div.ui-content'), self = this;
		view.willShow(ct);
		w.show();
		AnimManager.two(w, 'transition-fade-in', dlg, 'ui-popup-custom-in', function() {
			w.removeClass('transition-fade-in');
			dlg.removeClass('ui-popup-custom-in');
			view.didShow(ct);
		});
		// make the dialog center in screen
		dlg.css({ 'margin-top': '-' + Math.round(dlg.outerHeight() / 2) + 'px' });
	},
	popupWrapperHideForCustomView: function(view, id) {
		if (!view) return;
		id = id || view.__popupWrapperId;
		if (!id) return;
		view.willHide();
		var w = $('#' + id),
			dlg = w.find('div.ui-popup-custom');
		AnimManager.two(w, 'transition-fade-out', dlg, 'ui-popup-custom-out', function() {
			w.remove();
			view.didHide();
		});
		this.__popupWrapperCustomViewMap[id] = null;
		view.__popupWrapperId = null;
	},
	popupToast: function(message, jqCtEl, ellapse) {
		var offset = jqCtEl ? jqCtEl.offset() : { left: 0, top: 0 },
			offsetW = jqCtEl ? jqCtEl.outerWidth() : Layout.windowWidth,
			offsetH = jqCtEl ? jqCtEl.outerHeight() : Layout.windowHeight,
			preferedW = Math.min(280, Math.max(offsetW - 40, 100)),
			t = $('<div class="ui-popup-toast dim" style="max-width:' + preferedW + 'px">' + message + '</div>').appendTo($('body')),
			w = t.outerWidth(), h = t.outerHeight();
		// position it
		t.css({
			left: Math.min(Math.max(Math.round((offsetW - w) / 2) + offset.left, 0), Layout.windowWidth - w),
			top: Math.min(Math.max(Math.round((offsetH - h) / 2) + offset.top, 0), Layout.windowHeight - h)
		});
		AnimManager.one(t.removeClass('dim'), 'transition-fade-in', function() {
			setTimeout(function() {
				AnimManager.one(t, 'transition-fade-out', function() {
					t.remove();
				});
			}, ellapse || 1200);
		});
	}
};
Callback.popupCancel = function(btnId) {
	var _btn = $('#' + btnId),
		id = _btn.attr('data-rel'),
		view = Widget.__popupWrapperCustomViewMap[id];
	if (!view) return;
	if (view.onCancel && view.onCancel() === false)
		return;
	Widget.popupWrapperHideForCustomView(view, id);
};
Callback.popupOk = function(btnId) {
	var _btn = $('#' + btnId),
		id = _btn.attr('data-rel'),
		view = Widget.__popupWrapperCustomViewMap[id];
	if (!view) return;
	if (view.onOK && view.onOK() === false)
		return;
	Widget.popupWrapperHideForCustomView(view, id);
};


// ------------------------------------------------------------------------
// PanelView foundation
// 
Views.PanelView = {
	options: {
		tmpl: '',
		id: '',  // 为了统计需要，不输入id的话，默认跟tmpl相同
		name: 'PanelView基类'  // 为了统计需要，没有name不计入统计
	},

	// If this view has layout
	layout: null,

	willShow: function(ct) {
		this.render(ct);
		// View scroller or other view initialization here
	},

	didShow: function(ct) {
		if (this.layout) this.layout.didShow(ct);
	},

	willHide: function() {
		if (this.layout) this.layout.willHide();
	},

	didHide: function() {
		this.viewScrollerDestroy();
		if (this.layout) {
			this.layout.didHide();
			this.layout = null;
		}
	},

	onResize: function(w, h) {
		if (this.layout) this.layout.onResize(w, h);
	},

	render: function(ct, data) {
		Util.log('Rendering panel view using tmpl: ' + this.options.tmpl + '.html');
		var opts = this.options;
		ct.html(Views.tmpl[opts.tmpl](data || {}));
		if (Util.statisticEnabled && opts.name) {
			// 发送页面统计
			Util.statistic.onPageOpen(opts.id || opts.tmpl, this.options.name);
		}
	},

	$super: function() {
		var s = Views.PanelView, self = this;
		return {
			willShow: function() {
				s.willShow.call(self);
			},
			didShow: function() {
				s.didShow.call(self);
			},
			willHide: function() {
				s.willHide.call(self);
			},
			didHide: function() {
				s.didHide.call(self);
			},
			onResize: function(w, h) {
				s.onResize.call(self, w, h);
			}
		};
	},

	// Make scrollable
	_viewScroller: null,
	viewScrollerDestroy: function() {
		if (this._viewScroller) {
			this._viewScroller.destroy();
			this._viewScroller = null;
		}
	},
	viewScrollerInit: function(jqScroller, options) {
		if (this._viewScroller)
			this._viewScroller.destroy();
		this._viewScroller = Util.makeScrollable(jqScroller, options);
	},
	viewScrollerRefresh: function() {
		if (this._viewScroller) this._viewScroller.refresh();
	},
	viewScrollToElement: function(el, time) {
		if (this._viewScroller) this._viewScroller.scrollToElement(el, time);
	}
};


// ------------------------------------------------------------------------
// StageManager
// 
var StageManager = {
	initialize: function() {
		this.layout = new Layout.StackPanelLayout($('#mainStage'), 'left-right-ease');
	},

	pushView: function(panelView, transitionEndCallback) {
		this.layout.pushStackView(panelView, transitionEndCallback);
	},

	popView: function() {
		if (this.layout.getStackSize() <= 1) return;
		var currentView = this.getCurrentView();
		if (currentView.onPageBack && currentView.onPageBack() === false) {
			return;
		}
		this.layout.popStackView();
	},

	getCurrentView: function() {
		return this.layout ? this.layout.getCurrentStackView() : null;
	},

	// *hack* - 更改当前的页面Stack配置，新的viewStack最后一个页面必须和当前页面匹配
	fixViewStack: function(viewStack) {
		this.layout.fixViewStack(viewStack);
	},

    fixPageBack: function (viewStack) {
        this.layout.fixPageBack(viewStack);
    },

	delView: function (num) {
		this.layout.delView(num);
	},

    clearViewStack: function () {
        this.layout.clearViewStack();
    },

	debugViewStack: function() {
		this.layout.debugViewStack();
	}
};
Callback.showPage = function(btnId) {
	var _btn = $('#'+btnId),
		pageName = _btn.attr('data-rel');
	StageManager.pushView(Views[pageName]);
};
Callback.pageBack = function(btnId) {
	if (APPVERSION !== 'wechat' && mobile_system === 'android' && StageManager.getCurrentView().options.tmpl == "mainView") {
		// 在android系统上，首页按“回退”键退出系统
		Util.confirm('您确定要退出' + APPNAME + '吗？', function(index) {
			if (index === 2) {
				if (Util.statisticEnabled) Util.statistic.onAppExit();
//				UserProfile.setUserLogout();
				navigator.app.exitApp();
			}
		});
		return;
	}
	if (StageManager.getCurrentView().options.tmpl == "consultDetailView") {
		if ($("#image_mask").css('display') !== 'none') {
			Callback.closePic1();
			return;
		}
	}
	StageManager.popView();
};
