// ------------------------------------------------------------------------
// TabbedPanelLayout
// 
Layout.TabbedPanelLayout = Class.extend(Class.create(), Layout.LayoutBase, {
	fxRegistry: {
		'left-right': {
			'out' : ['ui-stage-moveToLeft', 'ui-stage-moveToRight'],
			'in'  : ['ui-stage-moveFromRight', 'ui-stage-moveFromLeft']
		},
		'left-right-fade': {
			'out' : ['ui-stage-moveToLeftFade', 'ui-stage-moveToRightFade'],
			'in'  : ['ui-stage-moveFromRight', 'ui-stage-moveFromLeft']
		},
		'left-right-ease': {
			'out' : ['ui-stage-moveToLeftEase', 'ui-stage-moveToRightEase'],
			'in'  : ['ui-stage-moveFromRightEase', 'ui-stage-moveFromLeftEase']
		},
		'top-bottom': {
			'out' : ['ui-stage-moveToTop', 'ui-stage-moveToBottom'],
			'in'  : ['ui-stage-moveFromBottom', 'ui-stage-moveFromTop']
		},
		'top-bottom-fade': {
			'out' : ['ui-stage-moveToTopFade', 'ui-stage-moveToBottomFade'],
			'in'  : ['ui-stage-moveFromBottom', 'ui-stage-moveFromTop']
		},
		'top-bottom-ease': {
			'out' : ['ui-stage-moveToTopEase', 'ui-stage-moveToBottomEase'],
			'in'  : ['ui-stage-moveFromBottomEase', 'ui-stage-moveFromTopEase']
		},
		'fade-in-out': {
			'out' : ['transition-fade-out', 'transition-fade-out'],
			'in'  : ['transition-fade-in', 'transition-fade-in']
		}
	},

	// ctor
	initialize: function(ct, fx) {
		this.transitionClass = this.fxRegistry[fx ? fx : 'left-right'];
		this.tabViewStack = [];

		ct.html(Views.tmpl.tabbedPanelLayout());
		var stageTabs = ct.find('div.ui-stage-tab');
		this.jqStageTabs = [
			stageTabs.eq(0),
			stageTabs.eq(1)
		];
		this.currentStageTabIndex = 0;
		this.currentTabViewIndex = -1;
	},

	willHide: function() {
		$.each(this.tabViewStack, function(_, tabView) {
			if(tabView) tabView.willHide();
		});
	},

	didHide: function() {
		$.each(this.tabViewStack, function(_, tabView) {
			if(tabView) tabView.didHide();
		});
		this.tabViewStack = [];
		this.jqStageTabs = null;
		this.currentStageTabIndex = 0;
		this.currentTabViewIndex = -1;
	},

	onResize: function(w, h) {
		$.each(this.tabViewStack, function(_, tabView) {
			tabView.onResize(w, h);
		});
	},

	addTabView: function(tabView) {
		this.tabViewStack.push(tabView);
	},

	getCurrentTabView: function() {
		return this.currentTabViewIndex !== -1 ? this.tabViewStack[this.currentTabViewIndex] : null;
	},

	showTabView: function(tabIndex, transitionEndCallback) {
		if (tabIndex < 0 || tabIndex >= this.tabViewStack.length)
			return;

		var nextStageTabIndex = this.currentTabViewIndex === -1 ? 0 : (this.currentTabViewIndex === tabIndex ? this.currentStageTabIndex : (this.currentStageTabIndex+1)%2),
			jqStageTab = this.jqStageTabs[nextStageTabIndex],
			newTabView = this.tabViewStack[tabIndex];

		if (tabIndex === this.currentTabViewIndex) {
			if (newTabView.onRefresh) newTabView.onRefresh();
			if (transitionEndCallback) transitionEndCallback();
			return;
		}

		if (this.currentTabViewIndex === -1 || this.currentTabViewIndex === tabIndex) {
			this.currentTabViewIndex = tabIndex;

			newTabView.willShow(jqStageTab);
			newTabView.didShow(jqStageTab);

			if (transitionEndCallback) transitionEndCallback();
			return;
		}

		var currentTabView = this.tabViewStack[this.currentTabViewIndex],
			classIndex = tabIndex - this.currentTabViewIndex > 0 ? 0 : 1,
			jqStageTabOut = this.jqStageTabs[this.currentStageTabIndex],
			self = this;
		// put forward...
		this.currentTabViewIndex = tabIndex;
		this.currentStageTabIndex = nextStageTabIndex;

		currentTabView.willHide();
		newTabView.willShow(jqStageTab);

		Views.isAnyViewInTransition = true;
		AnimManager.two(jqStageTabOut, this.transitionClass['out'][classIndex],
			jqStageTab.removeClass('ui-hidden'), this.transitionClass['in'][classIndex],
			function() {
				Views.isAnyViewInTransition = false;

				jqStageTabOut.addClass('ui-hidden');
				currentTabView.didHide();
				jqStageTabOut.html('');
				newTabView.didShow(jqStageTab);

				if (transitionEndCallback) transitionEndCallback();
			}
		);
	},

	clearContent: function() {
		$.each(this.tabViewStack, function(_, tabView) {
			tabView.willHide();
			tabView.didHide();
		});
		$.each(this.jqStageTabs, function(_, tab) {
			tab.html('');
		});
		this.currentStageTabIndex = 0;
		this.currentTabViewIndex = -1;
	}
});


// ------------------------------------------------------------------------
// StackPanelLayout
// 
Layout.StackPanelLayout = Class.extend(Class.create(), Layout.TabbedPanelLayout, {
	pushStackView: function(stackView, transitionEndCallback) {
		this.tabViewStack.push(stackView);
		var index = this.tabViewStack.length - 1;
		Layout.StackPanelLayout.__super__().showTabView.call(this, index, transitionEndCallback);
	},

	popStackView: function(transitionEndCallback) {
		if (this.currentTabViewIndex === 0) {
			var lastView = this.tabViewStack.pop();
			lastView.willHide();
			lastView.didHide();
			this.jqStageTabs[this.currentStageTabIndex].html('');
			this.currentTabViewIndex = -1;
			if (transitionEndCallback) transitionEndCallback();
			return lastView;
		}
		Layout.StackPanelLayout.__super__().showTabView.call(this, this.currentTabViewIndex - 1, transitionEndCallback);
		return this.tabViewStack.pop();
	},

	getCurrentStackView: function() {
		return this.currentTabViewIndex !== -1 ? this.tabViewStack[this.currentTabViewIndex] : null;
	},

	getStackSize: function() {
		return this.tabViewStack.length;
	},

	fixViewStack: function(viewStack) {
		var self = this;
		setTimeout(function() {
			if (!viewStack) {
				console.error('StackPanelLayout.fixViewStack >> the given viewStack is null.');
				return;
			}
			if (self.tabViewStack.last() !== viewStack.last()) {
				console.error('StackPanelLayout.fixViewStack >> the last view of the given viewStack does not match current view.');
				return;
			}
			self.tabViewStack = viewStack;
			self.currentTabViewIndex = viewStack.length - 1;
		}, 0);  // yield some time to make console.error happy
	},
	delView: function (num) {
		var self = this;
		num = num || 1;
		setTimeout(function() {
			if (self.tabViewStack.length < num) {
				console.error('num  moren than  tabViewStack.length.');
				return;
			}

			self.tabViewStack = self.tabViewStack.slice(0 , self.tabViewStack.length - num);
			self.currentTabViewIndex = self.tabViewStack.length - 1;
		}, 0);  // yield some time to make console.error happy
	},
    fixPageBack: function (viewStack) {
        var self = this;
        setTimeout(function() {
            if (!viewStack) {
                console.error('StackPanelLayout.fixPageBack >> the given viewStack is null.');
                return;
            }

            if (viewStack === self.tabViewStack.last()) {
                return;
            }
            var last = self.tabViewStack.pop();
            self.tabViewStack = self.tabViewStack.concat(viewStack, last);


            self.currentTabViewIndex = self.tabViewStack.length - 1;
        }, 0);  // yield some time to make console.error happy
    },
    clearViewStack: function () {
        var self = this;
        setTimeout(function () {
            self.tabViewStack = [Views.mainView];
            self.currentTabViewIndex = 0;
        }, 0);

    },

	debugViewStack: function() {
		var stack = [];
		$.each(this.tabViewStack, function(_, tabView) {
			var opt = tabView.options || {};
			stack.push(opt.id || opt.tmpl);
		});
		Util.log('Current view stack: [' + stack + ']');
	},
	
	// ----------------------------- Inherited but not implemented

	addTabView: function() {
		throw 'Not implemented';
	},

	getCurrentTabView: function() {
		throw 'Not implemented';
	},

	showTabView: function() {
		throw 'Not implemented';
	}
});
