// 在Callback上注册回调方法。
// 有下面几种方式会从Callback里调用回调方法：
// - 在一个带有.ui-btn的css class name的元素上点击，区分两种情况：
//	 a. 如果想在鼠标点下去或者触摸屏上手指点上去的时候直接执行，在元素上加上data-downAction="<YourCallbackFunc>",
//      之后鼠标或者手指放开的时候会触发data-upAction
//	 b. 如果想在鼠标释放或者触摸屏上手指移开才执行，在元素上加上 data-action="<YourCallbackFunc>"
//   c. 如果要长按触发某个时间，在元素上加上data-longTapAction="<YourCallbackFunc>"
// (参考 bindEvents 方法)
var Callback = {};
var wndw = $(window);

// 通用方法
var Util = {
    MILLIS_1_SEC: 1000,
    MILLIS_1_MIN: 60000,
    MILLIS_10_MIN: 600000,
    MILLIS_HALF_HOUR: 1800000,
    MILLIS_1_HOUR: 3600000,
    MILLIS_1_DAY: 86400000,
    touchStartX: 0,
    touchStartY: 0,

    calculateAngle: function (startPoint, endPoint) {
        var x = startPoint.x - endPoint.x;
        var y = endPoint.y - startPoint.y;
        var r = Math.atan2(y, x); //radians
        var angle = Math.round(r * 180 / Math.PI); //degrees

        //ensure value is positive
        if (angle < 0) {
            angle = 360 - Math.abs(angle);
        }

        return angle;
    },
    calculateDirection: function (startPoint, endPoint) {
        var angle = this.calculateAngle(startPoint, endPoint);

        if ((angle <= 45) && (angle >= 0)) {
            return 'left';
        } else if ((angle <= 360) && (angle >= 315)) {
            return 'left';
        } else if ((angle >= 135) && (angle <= 225)) {
            return 'right';
        } else if ((angle > 45) && (angle < 135)) {
            return 'down';
        } else {
            return 'up';
        }
    },
    inAppBrowserRef: null,
    debug: APPVERSION !== 'product',
    log: function (msg) {
        if (!this.debug)
            return;
        if (typeof msg === 'object')
            msg = JSON.stringify(msg);
        console.log('=====>>>>> ' + msg + ' <<<<<=====');
    },
    rtime: function () {
        return (Math.floor(Math.random() * 10001)).toString() + (new Date().getTime()).toString();
    },
    isOnline: function () {
        this.log('检测网络状态：' + window.networkState);
        return (window.networkState !== 'No network connection' && window.networkState !== 'Unknown connection');
    },
    networkStateChanged: function (networkState) {
        if (this.isOnline())
            return;
        // TODO
    },
    loginSuggest: function () {
        if (!UserProfile.isUserLoggedIn()) {
            this.confirm("登录之后才能继续操作哦:-( 现在就去登录吗？", function (idx) {
                if (idx === 2) {
                    StageManager.pushView(Views.loginView);
                }
            });
            return false;
        } else if (!UserProfile.patients.length) {
            Widget.popupFailure("您尚未添加任何用户");
            Util.data('addMember', "");
            StageManager.pushView(Views.addMemberView);
            return false;
        } else {
            return true;
        }
    },
    data: function (name, data) {
        if (typeof data !== 'undefined' || arguments.length === 2) {
            console.log(arguments);
            localStorage.setItem(name, JSON.stringify(data));
        } else {
            data = localStorage.getItem(name);
            return data && data !== 'undefined' ? JSON.parse(data) : undefined;
        }
    },
    dataClear: function (name, data) {
        data = localStorage.getItem(name);
        localStorage.removeItem(name);
        return data ? JSON.parse(data) : undefined;
    },
    mobiscrollSelect: function (selector, options) {
        return $(selector).hide().mobiscroll().select($.extend({
            theme: 'ios7',
            lang: 'zh',
            display: 'bottom',
            mode: 'scroller',
            showInput: false
        }, options));
    },
    mobiscrollDate: function (selector, options) {
        return $(selector).mobiscroll().date($.extend({
            theme: 'ios7',
            lang: 'zh',
            dateOrder: 'yymmdd',
            dateFormat: 'yyyy-mm-dd',
            display: 'bottom',
            mode: 'scroller'
        }, options));
    },
    alert: function (msg, callback, title, buttonName) {
        if (!mobile_system || (!mobile_system && APPVERSION === 'web')) {
            alert(msg);
            if (callback) callback();
            return;
        }
        var _title = title ? title : '温馨提示', _callback = callback ? callback : function () {
        };
        _buttonName = buttonName ? buttonName : '确定';
        navigator.notification.alert(msg, _callback, _title, _buttonName);
    },
    confirm: function (msg, callback, title, buttonLabels) {
        if (!mobile_system || (!mobile_system && APPVERSION === 'web')) {
            var _r = confirm(msg) ? 2 : 1;
            setTimeout(function () {
                callback(_r);
            }, 0);
            return;
        }
        var _title = title ? title : '请确认', _callback = callback ? callback : function (index) {
        };
        _buttonLabels = buttonLabels ? buttonLabels : '取消,确定';
        navigator.notification.confirm(msg, callback, _title, _buttonLabels);
    },
    beep: function (n) {
        if (!mobile_system && APPVERSION === 'web')
            return;
        var _n = n ? n : 2;
        navigator.notification.beep(_n);
        navigator.notification.vibrate(2000);
    },
    trim: function (str) {
        return str.replace(/(^\s*)|(\s*$)/g, "");
    },
    textLength: function (str) {
        return str.replace(/[^\x00-\xff]/g, "aa").length;
    },
    _autoID: 0,
    getAutoID: function () {
        this._autoID = this._autoID + 1;
        return '_id_' + this._autoID.toString();
    },

    isLogin: function () {
        return Util.loginSuggest();
    },

    // Update相关
    checkUpdateOrNewInstall: function (callback) {
        Util.log('Current version: ' + VERSION_NUMBER);
        var lastVersion = Util.data('version.lastVersion');
        if (!lastVersion) {
            Util.data('version.lastVersion', VERSION_NUMBER);
            if (callback) callback({ isNewInstall: true });
        } else if (lastVersion !== VERSION_NUMBER) {
            Util.data('version.lastVersion', VERSION_NUMBER);
            if (callback) callback({ isUpdate: true, lastVersion: lastVersion });
        } else {
            if (callback) callback({ isNewInstall: false, isUpdate: false });
        }
    },

    checkVersionUpdate: function (appInit) {
        function ajaxDoneHandle(json) {
            console.log(json);
            if (json.success) {
                UserProfile.isCheck = true;
                var $ele = $('#dialog'), _tmpl;
                if (!$ele.length) {
                    json.data.content = (json.data.content || "").split("\\n");
                    _tmpl = Views.tmpl.versionCheckView({data: json.data});
                    $('.ui-page').after(_tmpl);
                }
                Util.makeDialog("dialog");
                $('#downNewVersion').attr('data-mustUpgrade', json.data.mustUpgrade);
            }
        }
        if (!UserProfile.isCheck) {
            DataExchange.getNewVersion(appInit).done(ajaxDoneHandle);
        }
    },

    makeDialog: function (ele, inPage) {
        var _dialog = $('#' + ele);
        _dialog.show().prev().css({'opacity': 0.5});
        var cancelDialog = function (e) {
            var _target = e.srcElement, $target = $(_target), btn = $target.hasClass('click'), action, btnId;

            if (inPage) {
                if (_dialog.children().first().has(_target).length && !btn) {
                    return;
                }
            } else {
                if (!btn) {
                    return;
                }
            }

            _dialog.hide().prev().css({'opacity': 1});
            document.removeEventListener('click', cancelDialog);
            e.stopPropagation();

            if (btn) {
                action = $target.attr('data-action'), btnId = $target.attr('id');
                Callback[action](btnId);
            }
        };
        document.addEventListener('click', cancelDialog);
    },

    // Make scrollable
    makeScrollable: function (jqScroller, options) {
        return new iScroll(jqScroller[0], $.extend({
            checkDOMChanges: true,
            snap: false,
            momentum: true,
            /*bounce: false,*/ // Setting this value to false to make it behave like in android
            scrollbarClass: 'myScrollbar',
            fadeScrollbar: true,
            hideScrollbar: true,
            onBeforeScrollStart: function (e) {
                var target = e.target;
                while (target.nodeType !== 1)
                    target = target.parentNode;
                var _tagNameLc = target.tagName.toLowerCase();
                if (_tagNameLc !== 'select' && _tagNameLc !== 'input' && _tagNameLc !== 'textarea' && _tagNameLc !== 'option')
                    e.preventDefault();
            },
            onScrollMove: function (e) {
                Views.isAnyViewScrolling = true;
            },
            onScrollEnd: function (e) {
                Views.isAnyViewScrolling = false;
            },
            onTouchEnd: function () {
                var touchend = $(this.wrapper).attr('data-touchend');
                if (touchend) Callback[touchend]();
            }
        }, options || {}) // $.extend
        );
    },

    makeScrollableWithPullRefresh: function (jqScroller, delegate, options) {
        return new iScroll(jqScroller[0], $.extend({
            checkDOMChanges: true,
            topOffset: 38,
            hScrollbar: false,
            vScrollbar: false,
            onRefresh: function () {
                var el = document.getElementById('pullDown'), el2 = document.getElementById('pullUp');
                if (el && el.className.match('ui-pulldown loading')) {
                    el.className = 'ui-pulldown';
                    el.querySelector('.pullDownLabel').innerHTML = '下拉立即刷新';
                }
                if (el2 && el2.className.match('ui-pullUp loading')) {
                    el2.className = 'ui-pullUp';
                    el2.querySelector('.pullup-inner').innerHTML = '向上加载更多';
                }
            },
            onBeforeScrollStart: function (e) {
                var target = e.target;
                while (target.nodeType !== 1)
                    target = target.parentNode;
                var _tagNameLc = target.tagName.toLowerCase();
                if (_tagNameLc !== 'select' && _tagNameLc !== 'input' && _tagNameLc !== 'textarea' && _tagNameLc !== 'option')
                    e.preventDefault();
            },
            onScrollMove: function () {
                Views.isAnyViewScrolling = true;
                var el = document.getElementById('pullDown'), el2 = document.getElementById('pullUp');
                if (isNaN(this.y)) {
                    el.className = 'ui-pulldown flip';
                    el.querySelector('.pullDownLabel').innerHTML = '松开立即刷新';
                    this.minScrollY = 0;
                    return;
                }
                if (this.y > 5) {
                    el.className = 'ui-pulldown flip';
                    el.querySelector('.pullDownLabel').innerHTML = '松开立即刷新';
                    this.minScrollY = 0;
                } else if (this.y > -38 && this.y < 5) {
                    el.className = 'ui-pulldown';
                    el.querySelector('.pullDownLabel').innerHTML = '下拉立即刷新';
                    this.minScrollY = -38;
                } else if (el2 && this.y < this.wrapperH - this.scrollerH - 38 && !el2.className.match('ui-pullUp loading') && !el2.className.match('ui-pullUp ui-hidden')) {
                    el2.className = 'ui-pullUp loading';
                    el2.querySelector('.pullup-inner').innerHTML = '';
                    if (delegate.onPullUpRefresh) delegate.onPullUpRefresh();
                }
            },
            onScrollEnd: function () {
                Views.isAnyViewScrolling = false;
                var el = document.getElementById('pullDown');
                if (el.className.match('ui-pulldown flip')) {
                    el.className = 'ui-pulldown loading';
                    el.querySelector('.pullDownLabel').innerHTML = '加载中...';
                    if (delegate.onPullDownRefresh) delegate.onPullDownRefresh();
                }
            }
        }, options || {}) // $.extend
        );
    },

    pagehScrollInit: function (jqEl, indicatorChangeCallback) {
        return new iScroll(jqEl[0], {
            snap: true,
            momentum: false,
            hScrollbar: false,
            onBeforeScrollStart: function (e) {
                var target = e.target;
                while (target.nodeType !== 1)
                    target = target.parentNode;
                var _tagNameLc = target.tagName.toLowerCase();
                if (_tagNameLc !== 'select' && _tagNameLc !== 'input' && _tagNameLc !== 'textarea' && _tagNameLc !== 'option') {
                    $('input').blur();
                    e.preventDefault();
                }
            },
            onScrollMove: function (e) {
                Views.isPageScrolling = true;
            },
            onScrollEnd: function () {
                Views.isPageScrolling = false;
                if (indicatorChangeCallback) indicatorChangeCallback(this.currPageX);
            },
            onTouchEnd: function () {
            }
        });
    },

    _destructors: [],
    registerDestructor: function (dtor) {
        this._destructors.push(dtor);
    },

    babyDateSelectInit: function (selector) {
        var now = new Date();
        var year = now.getFullYear();
        $('#basicUsageShow').click(function () {
            $(selector).mobiscroll('show');
            return false;
        });
        var baby = LocalMode('baby').get();
        return $(selector).mobiscroll().date({
            theme: 'ios7',
            lang: 'zh',
            dateOrder: 'yymmdd',
            dateFormat: 'yyyy-mm-dd',
            display: 'bottom',
            mode: 'scroller',
            defaultValue: $.trim($(selector).text()) ? new Date($(selector).text()) : new Date(),
            startYear: baby.birthday.split('-')[0],
            endYear: year,
            onSelect: function (value, inst) {
                var select = new Date(value);
                var selectDate = DateUtil.formatYYMMDD(select);
                var nowFormat = DateUtil.formatYYMMDD(now);
                var birthDay = DateUtil.formatYYMMDD(new Date(baby.birthday));
                if (selectDate - nowFormat > 0) {
                    Widget.popupFailure("选择日期不能晚于今天");
                    return false;
                }
                if (selectDate - birthDay < 0) {
                    Widget.popupFailure("选择日期不能早于出生日期");
                    return false;
                }
                $(selector).text(value);
            }
        });

    },

    birthDaySelectInit: function () {
        var now = new Date();
        var year = now.getFullYear();
        $('#birth').mobiscroll().date({
            theme: 'ios7',
            lang: 'zh',
            dateOrder: 'yymmdd',
            dateFormat: 'yyyy-mm-dd',
            dayText: '日',
            monthText: '月',
            yearText: '年',
            display: 'bottom',
            mode: 'scroller',
            defaultValue: $.trim($('#birth').text()) ? new Date($('#birth').text()) : new Date(),
            startYear: year - 90,
	        maxDate: now,
            onSelect: function (value, inst) {
                $('#birth').val(value);
            }
        });
        $('#birth').click(function () {
            $('#birth').mobiscroll('show');
            return false;
        });
    },
    mobiscrollDateTime: function (selector, options) {
        var now = new Date(), year = now.getFullYear();
        return $(selector).mobiscroll().datetime($.extend({
            theme: 'ios7',
            lang: 'zh',
            dateOrder: 'yymmdd',
            dateFormat: 'yyyy-mm-dd',
            timeFormat: 'HH:ii',
            display: 'bottom',
            mode: 'scroller',
            maxDate: now,
            showNow: true,
            startYear: year - 10,
            onSelect: function (value, inst) {
                $(selector).val(value);
            }
        }, options));
    },

    draw: function () {
        var canvas = null;

        function initCanvas(btnId) {
            var _btn = document.getElementById(btnId);
            if (_btn.getContext) {
                canvas = _btn.getContext('2d');
            }
        };

        function setCanvas(options) {
            canvas = $.extend(canvas, options);

        };
        function getCanvas() {
            return canvas;
        };
        function draw(callback, data) {
            canvas.beginPath();
            callback(data, canvas);
            canvas.stroke();
        };
        function clear() {
            canvas = null;
        };
        function drawDashed(data, options) {
            canvas.beginPath();
            var height = options.canvasHeight, minUnitY = height / options.diffY, i = 0, deltaX = options.minUnitX * 1, x0 = 0, y0 = 0, len = data.length, j, deltaY, numDashes, x, y, addX, addY;
            for (i; i < len; i++) {
                deltaY = (data[i + 1] - data[i]) * minUnitY;
                numDashes = Math.floor(Math.sqrt(deltaX * deltaX + deltaY * deltaY) / options.dashLength);
                x = deltaX / numDashes;
                y = deltaY / numDashes;
                for (j = 0; j < numDashes; j++) {
                    addX = x * j;
                    addY = y * j;
                    if (j % 2 == 0) {
                        x0 = deltaX * i;
                        y0 = data[i] * minUnitY;
                        canvas.moveTo(x0 + addX, height - (y0 + addY));
                    } else {
                        canvas.quadraticCurveTo(x0 + addX / 2, height - (y0 + addY / 2), x0 + addX, height - (y0 + addY));
                    }
                }
            }
            canvas.stroke();
        };
        function drawTrend(data, options) {
            var len = data.length;
            if (!len) {
                return;
            }
            canvas.beginPath();
            //390-X是因为画布的底部是在最上面，实际中底部在下面，画布高度是390，所以减去
            //x*40是因为x间隔是40
            var minX = options.minUnitX, h = options.canvasHeight, r = options.cicleR, minY = options.minY, maxY = options.maxY, unitY = h / (maxY - minY), x, y, i;
            data[0].y = data[0].y - minY;
            canvas.moveTo(data[0].x * minX, h - data[0].y * unitY);
            for (i = 0; i < len; i++) {
                data[i].y -= minY;
                x = data[i].x * minX;
                y = h - data[i].y * unitY;
                canvas.lineTo(x, y);
                canvas.arc(x, y, r, 0, 2 * Math.PI, true);
                canvas.moveTo(x, y);
            }
            canvas.stroke();
        }

        return {
            initCanvas: initCanvas,
            setCanvas: setCanvas,
            getCanvas: getCanvas,
            draw: draw,
            drawDashed: drawDashed,
            clear: clear,
            drawTrend: drawTrend
        };
    },
    //血压趋势
    canvasInit: function (canvas) {
        var vcContext = canvas.getContext("2d"),
            devicePixelRatio = window.devicePixelRatio || 1,
            backingStoreRatio = vcContext.webkitBackingStorePixelRatio
                || vcContext.mozBackingStorePixelRatio
                || vcContext.msBackingStorePixelRatio
                || vcContext.oBackingStorePixelRatio
                || vcContext.backingStorePixelRatio || 1,
            ratio = devicePixelRatio / backingStoreRatio;

        // upscale the canvas if the two ratios don't match
        if (devicePixelRatio !== backingStoreRatio) {
            var oldWidth = canvas.width, oldHeight = canvas.height;

            canvas.width = oldWidth * ratio;
            canvas.height = oldHeight * ratio;

            canvas.style.width = oldWidth + 'px';
            canvas.style.height = oldHeight + 'px';

            // now scale the context to counter
            // the fact that we've manually scaled
            // our canvas element
            vcContext.scale(ratio, ratio);
        }
        return vcContext;
    }
};

// ------------------------------------------------------------------------
// UserProfile
var UserProfile = {
    loginState: false,
    id: 0,
    familyId: 0,
    babies: null,
    code: '',
    mobile: '',
    nickname: '',
    email: '',
    sourceId: 0,
    patients: null,
    isCheck: false,

    findMember: function (id) {
        var i = 0;
        for (; i < UserProfile.patients.length; i++)
            if (UserProfile.patients[i].id === id - 0)
                return UserProfile.patients[i];
        return null;
    },

    getMember: function (memberId) {
        if (!memberId) {
            return {};
        }
        var ps = UserProfile.patients, i;
        for (i = 0; i < ps.length; i++) {
            if (ps[i].memberId == memberId) {
                return ps[i];
            }
        }
        return {};
    },
    updateLocalMode: function (options) {
        var Maternal = LocalMode("maternal"), Baby = LocalMode("baby"), maternal = Maternal.get(), baby = Baby.get();
        if (options.id == maternal.id && options.role !== "GRAVIDE") {
            Maternal.remove();
        }
        if (options.id == baby.id && options.role != "CHILD") {
            Baby.remove();
        }
    },
    addLocalMode: function (mode, data) {
        var Mode = LocalMode(mode);
        Mode.remove();
        Mode.update(data);
    },

    getMemberById: function (id) {
        if (!id) {
            return {};
        }
        var ps = UserProfile.patients, i;
        for (i = 0; i < ps.length; i++) {
            if (ps[i].id == id) {
                return ps[i];
            }
        }
        return {};
    },

    delMemberById: function (id) {
        if (!id) {
            return;
        }
        var ps = UserProfile.patients, i;
        for (i = 0; i < ps.length; i++) {
            if (ps[i].id == id) {
                break;
            }
        }
        this.patients.splice(i, 1);
        this.delLocalMode(id);
    },

    delLocalMode: function (patientId) {
        var Maternal = LocalMode('maternal'), maternal = Maternal.get(), Baby = LocalMode('baby'), baby = Baby.get();
        if (maternal && maternal.id === Number(patientId)) {
            Maternal.remove();
        }
        if (baby && baby.id === Number(patientId)) {
            Baby.remove();
        }
        if (Util.data('current.member').id == Number(patientId)) {
            Util.data('current.member', UserProfile.patients[0]);
        }
    },

    updateCurrentMember: function (memberId) {
        for (var i = 0, len = UserProfile.patients.length; i < len; i++) {
            if (UserProfile.patients[i].memberId === Number(memberId)) {
                Util.data('current.member', UserProfile.patients[i]);
                return;
            }
        }
        Util.data('current.member', '');
    },
    getFamilyId: function () {
        return this.familyId;
    },
    getBabies: function () {
        if (this.babies) {
            return this.babies;
        }
        var babies = [];
        for (var i = 0, len = UserProfile.patients.length; i < len; i++) {
            var monthAge = UserProfile.patients[i].monthAge;
            if (UserProfile.patients[i].role === "CHILD") {
                if (monthAge <= 24) {
                    UserProfile.patients[i].monthAgeStr = monthAge + '月';
                } else {
                    UserProfile.patients[i].monthAgeStr = Math.floor(monthAge / 12) + '岁';
                }
                babies.push(UserProfile.patients[i]);
            }
        }
        this.babies = babies;
        LocalMode('baby').update(babies[0]);
        return this.babies;
    },
    setUserLoggedIn: function () {
        this.loginState = true;
        this.getBabies();
        this.loginLocalMode(); //当在不同的手机上登陆时，更新本手机本地存储数据
//		Util.data('profile.user', {
//			id: this.id,
//			code:this.code,
//			mobile:this.mobile,
//			nickname: this.nickname,
//			email: this.email,
//			sourceId: this.sourceId,
//			familyId:this.familyId,
//			patients: this.patients,
//			babies:this.babies
//		});
//		Util.data('profile.lastLoginTime', new Date().getTime());
    },

    loginLocalMode: function () {
        var Maternal = LocalMode("maternal"), Baby = LocalMode("baby"), maternal = Maternal.get(), baby = Baby.get(), ps = UserProfile.patients, i = 0, len = ps.length, maternalChange = true, babyChange = true, pwd = Util.data('setPwd');
        if (pwd && pwd.code !== UserProfile.code) {
            Util.data('setPwd', '');
        }
        for (; i < len; i++) {
            if (maternal.id === ps[i].id && ps[i].role === "GRAVIDE") {
                maternalChange = false;
            }
            if (baby.id === ps[i].id && ps[i].role === "CHILD") {
                babyChange = false;
            }
        }
        if (maternalChange) {
            Maternal.remove();
        }
        if (babyChange) {
            Baby.remove();
        }
    },
    setUserRemindPassword: function (params) {
        Util.data("remindPassword", params);
    },
    clearUserRemindPassword: function () {
        Util.data("remindPassword", "");
    },
//	clearUserAutoLogin: function() {
//		Util.data('profile.user', "");
//	},
    setUserLogout: function () {
        this.loginState = false;
        this.id = 0;
        this.code = '';
        this.mobile = '';
        this.nickname = '';
        this.email = '';
        this.patients = [];
        this.babies = null;
        this.isCheck = false;
//		Util.data('profile.user', '');//账户信息缓存
        //页面停留的位置
        Util.data('current.riskTag', '');//风险评估的标签
        Util.data('vsList', '');//体征数据
        Util.data('vsList.time', '');//多少时间的体征数据
        Util.data('vsList.lastGet', '');//体征数据读取时间
        Util.data('members.lastGet', '');//最新一次成员信息查询时间
        Util.data('current.member', '');//当前操作的成员缓存
        Util.data('abReminds', '');//异常提醒
        Util.data('abReminds.getTime', '');//异常提醒获取时间
        Util.data('abReminds.unreadNum', '');//异常提醒未读数
        // Util.data('devices', '');//设备信息
        Util.data('news.categories.timeStamp', 0);//新闻标题获取时间
        Util.data("consult.tab.index", 1);
        Util.data("account", "");
        Util.data("pwd", "");
        LocalMode('maternal').remove();
        LocalMode('baby').remove();

        if (Util.data('reGetSC.interval') || Util.data('reGetSC.time')) {
            clearInterval(Util.data('reGetSC.interval'));
            Util.data('reGetSC.interval', '');
            Util.data('reGetSC.time', '');
        }
    },

    isUserLoggedIn: function () {
        return this.loginState;
//		if ( this.loginState ){
//			return true;
//		}
//		if( Util.data('profile.user') ){
//			if((new Date().getTime()-Util.data('profile.lastLoginTime'))>DateUtil.ONE_WEEK_MILLSECONDS){//缓存保留一周
//				return false;
//			}
//			this.loginState = true;
//			$.extend(UserProfile,Util.data('profile.user'));
//			return true;
//		}
//		return false;
    },

    isRemindPwd: function () {
        if (Util.data("remindPassword")) {
            var params = Util.data('remindPassword');
            if (params.code && params.password) {
                return true;
            }
        }
        return false;
    },

    /*
     * 适用于用户选择新的妇女模块用户
     * memberId是选取用户的标识
     * 本地存储的memberId 不和 用户选取的一致则清空本地，并更新
     * */
    updateMaternal: function (memberId) {
        //更新本地存储的maternal模块
        var ps = this.patients, Maternal = LocalMode('maternal'), maternal = Maternal.get(), len = ps.length, i = 0;
        if (memberId != maternal.memberId) {
            Maternal.remove();
            Maternal.update({code: UserProfile.code});
        }
        for (; i < len; i++) {
            if (ps[i].memberId == memberId) {
                Maternal.update(ps[i]);
                break;
            }
        }
    },

    updateBaby: function (memberId) {
        var ps = this.patients, Baby = LocalMode('baby'), baby = Baby.get(), len = ps.length, i = 0;
        if (memberId != baby.memberId) {
            Baby.remove();
            Baby.update({code: UserProfile.code});
        }
        for (; i < len; i++) {
            if (ps[i].memberId == memberId) {
                var monthAge = ps[i].monthAge;
                if (monthAge <= 24) {
                    ps[i].monthAgeStr = monthAge + '月';
                } else {
                    ps[i].monthAgeStr = Math.floor(monthAge / 12) + '岁';
                }
                Baby.update(ps[i]);
                break;
            }
        }
    },

    /**
     * 当用户点击妇女模块的时候会进行检查
     * 1.用户是否登录，无则跳到登录页面
     * 2.判断本地存储的主账号code是否和登陆者一致，不一致清空本地
     * 3.判断账号内部是否有女性，无跳转到添加用户页面(性别:女)，有则正常转入
     * 注：当登录模块完成，部分功能需要迁移到登录模块
     * */
    checkMaternal: function () {
        if (!this.isUserLoggedIn()) {
            // 这里登录状态判断后续需要更改
            //TODO
            StageManager.pushView(Views['loginView']);
            return;
        }
        var Maternal = LocalMode('maternal'), maternal = Maternal.get(), ps = UserProfile.patients || Util.data('profile.user').patients, i = 0, len = ps.length;
        for (; i < len; i++) {
            if (ps[i].role === "GRAVIDE") {
                Maternal.update(ps[i]);
                StageManager.pushView(Views.maternalHealthCareView);
                return;
            }
        }
        if (UserProfile.code != maternal.code) {
            Maternal.remove();
        }
        Maternal.update({code: UserProfile.code});
        if (i == len || !len) {
            Widget.popupFailure("成员中未发现孕妇信息", function () {
                Util.data('addMember', 'maternal');
                StageManager.pushView(Views.addMemberView);
            });
        }
    },

    checkBaby: function () {
        if (!this.isUserLoggedIn()) {
            // 这里登录状态判断后续需要更改
            //TODO
            StageManager.pushView(Views['loginView']);
            return;
        }
        var Baby = LocalMode('baby'), baby = Baby.get(), ps = UserProfile.patients || Util.data('profile.user').patients, i = 0, len = ps.length;
        for (; i < len; i++) {
            if (ps[i].role === "CHILD") {
                var monthAge = ps[i].monthAge;
                if (monthAge <= 24) {
                    ps[i].monthAgeStr = monthAge + '月';
                } else {
                    ps[i].monthAgeStr = Math.floor(monthAge / 12) + '岁';
                }
                Baby.update(ps[i]);
                StageManager.pushView(Views.babyHealthCareView);
                return;
            }
        }
        if (UserProfile.code != baby.code) {
            Baby.remove();
        }
        Baby.update({code: UserProfile.code});
        if (i == len || !len) {
            Widget.popupFailure("用户组内未发现儿童信息", function () {
                Util.data('addMember', 'baby');
                StageManager.pushView(Views.addMemberView);
            });
        }
    }
};

// ------------------------------------------------------------------------
// Validator
//
var Validator = {
    phoneNo: /^\d{8,11}$/,
    mobileNo: /^1[3|4|5|7|8][0-9]\d{8}$/,
    email: /^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/,
    realName: /^[\u4e00-\u9fa5]{2,5}$/,
    idCard: /(^\d{15}$)|(^\d{18}$)|(^\d{17}(\d|X|x)$)/,
    medicalCardNo: /^[0-9a-zA-Z]+$/,
    password: /^(\w){6,16}$/,
    emoji: /[\uD800-\uDBFF][\uDC00-\uDFFF]/g,
    pass: true,
    reject: false,
    mc: function (mc) {
        if (isNaN(mc) || mc < 15 || mc > 45) {
            Widget.popupFailure("请选择正确的月经周期");
            return this.reject;
        }
        return this.pass;
    },
    empty: function (arg, msg) {
        if (!$.trim(arg)) {
            Widget.popupFailure(msg);
            return this.pass;
        }
        return this.reject;
    },
    cardHolder: function (args) {
        var len = args.length, i = 0;
        for (; i < len; i++) {
            if (!args[ i ]) {
                Widget.popupFailure("请选择持卡人");
                return this.reject;
            }
        }
        return this.pass;
    },
    hosId: function (hosId) {
        return !this.empty(hosId, "请先选择医院");
    },
    deptId: function (deptId) {
        return !this.empty(deptId, "请先选择科室");
    },
    cardHolderPhone: function (phone) {
        if (!phone) {
            Widget.popupFailure("您选择的持卡人未填写手机号");
            return this.reject;
        }
        return this.pass;
    },
    questionContent: function (question) {
        if (!this.empty(question, "请输入问题内容") && this.bettwenM_N(question.length, 10, 100, "问题描述请控制在10到100字之间，谢谢！")) {
            return this.pass;
        }
        return this.reject;
    },
    testMedicalCardNo: function (medicalCardNo) {
        if (!this.medicalCardNo.test(medicalCardNo)) {
            Widget.popupFailure("门诊号格式为字母和数字混合体");
            return this.reject;
        }
        return this.pass;
    },
    medicalCardValid: function (medicalCardValid) {
        if ($.trim(medicalCardValid) === "true") {
            Widget.popupFailure("您不可以更新有效就诊卡");
            return this.reject;
        }
        return this.pass;
    },
    authCode: function (authCode) {
        if (!authCode) {
            Widget.popupFailure("请输入验证码");
            return this.reject;
        }
        return this.pass;
    },
    testPwd: function (newPwd) {
        if (!this.password.test(newPwd)) {
            Widget.popupFailure("请输入正确的密码，6-16位");
            return  this.reject;
        }
        return this.pass;
    },
    validTwoPwd: function (options) {
        if (this.empty(options.oldPwd, "请输入旧密码") || this.empty(options.newPwd, "请输入新密码") || !this.testPwd(options.newPwd)) {
            return this.reject;
        }
        if (options.newPwds !== options.newPwd) {
            Widget.popupFailure("两次密码不一致");
            return this.reject;
        }
        return this.pass;
    },
    medCard2: function (args) {
        var len = args.length, i = 0;
        for (; i < len; i++) {
            if (!args[ i ]) {
                Widget.popupFailure("请填写门诊号");
                return this.reject;
            }
        }
        return this.pass;
    },
    medCard: function (args) {
        var len = args.length, i = 0;
        for (; i < len; i++) {
            if (!args[ i ]) {
                Widget.popupFailure("请填写就诊卡信息");
                return this.reject;
            }
        }
        return this.pass;
    },
    content: function (content) {
        var contentTrim = $.trim(content), len;
        if (this.empty(contentTrim, "请输入内容")) {
            return this.reject;
        }
        len = contentTrim.length;
        if (len > 100) {
            Widget.popupFailure("目前字数" + len + ",已超过100字");
            return this.reject;
        }
        return this.pass;
    },
    relative: function (options) {
        if (!options.name || !Validator.mobileNo.test(options.phoneNum)) {
            Widget.popupFailure("请输入正确的联系方式");
            return  this.reject;
        }
        return this.pass;
    },
    testEmail: function (email) {
        var status = this.pass;
        if (email && !Validator.email.test(email)) {
            Widget.popupFailure("请输入正确的邮箱");
            return this.reject;
        }
        return this.pass;
    },
    lmp: function (lmp) {
        return !this.empty(lmp, "请选择正确的月经开始时间");
    },
    memberId: function (memberId) {
        return !this.empty(memberId, "请选择用户");
    },
    weight: function (weight) {
        return this.bettwenM_N(weight, 2, 30, "请输入正确的体重,范围2-30");
    },
    height: function (height) {
        return this.bettwenM_N(height, 40, 120, "请输入正确的身高,范围40-120");
    },
    headCircum: function (headCircum) {
        return this.bettwenM_N(headCircum, 30, 50, "请输入正确的头围,范围30-50");
    },
    chestCircum: function (chestCircum) {
        return this.bettwenM_N(chestCircum, 30, 50, "请输入正确的胸围,范围30-50");
    },
    compareDate: function (start, end, msg) {
        if (new Date(end) < new Date(start)) {
            Widget.popupFailure(msg);
            return this.reject;
        }
        return this.pass;
    },
    birthday: function (start, end) {
        return this.compareDate(start, end, "输入日期早于出生日期");
    },
    now: function (start) {
        return this.compareDate(start, new Date(), "选择日期不能晚于今天");
    },
    bettwenM_N: function (arg, M, N, msg) {
        if (!(arg >= M && arg <= N)) {
            Widget.popupFailure(msg);
            return this.reject;
        }
        return this.pass;
    },
    validIdCard: function (options) {
        var idCard = options.testIdCard, _birth = options.testBirth, sex = options.testSex, idCardSex, idCardBirth, isOldCard = idCard.length === 15;
        idCardSex = ( isOldCard ? (idCard.slice(-1) % 2 ? '男' : '女') : (idCard.substr(-2, 1) % 2 ? '男' : '女') );
        if (idCardSex !== sex) {
            Widget.popupFailure("性别与身份证上不一致");
            return this.reject;
        }
        idCardBirth = ( isOldCard ? '19' + idCard.substring(6, 12) : idCard.substring(6, 14));
        if (idCard && _birth) {
            if (_birth !== idCardBirth) {
                Widget.popupFailure("出生日期与身份证上不一致");
                return this.reject;
            }
        }
        return true;
    },
    validator: function (options) {
        for (var key in options) {
            if (key === "checkIdCard") {
                if (!this.validIdCard(options)) {
                    return this.reject;
                }
                ;
            } else if (!this[key](options[key])) {
                return this.reject;
            }
        }
        return this.pass;
    },
    testMobileNo: function (phoneNo) {
        if (!this.mobileNo.test(phoneNo)) {
            Widget.popupFailure("请输入正确的手机号");
            return this.reject;
        }
        return this.pass;
    },
    testUsername: function (username) {
        if (this.empty(username, "请输入姓名")) {
            return this.reject;
        } else if (Util.textLength(username) > 10){
            Widget.popupFailure("姓名不多于10个字符");
            return this.reject;
        }
        return this.pass;
    },
    testBirth: function (birth) {
        return !this.empty(birth, "请输入出生日期");
    },
    testSex: function (sex) {
        if (!sex || "男女".indexOf(sex) === -1) {
            Widget.popupFailure("请选择正确的性别");
            return this.reject;
        }
        return this.pass;
    },
    testBaby: function (birth) {
        if ((new Date() - new Date(birth) ) / 1000 / 60 / 60 / 24 / 31 > 72) {
            Widget.popupFailure("请输入婴儿成员");
            return this.reject;
        }
        return this.pass;
    },
    testIdCard: function (idCard) {
        if (!idCard) {
            Widget.popupFailure("请输入身份证号");
            return this.reject;
        }
        if (!this.idCard.test(idCard)) {
            Widget.popupFailure("请输入正确的身份证");
            return this.reject;
        }
        return this.pass;
    },
    stringIsMobileNo: function (s) {
        return this.mobileNo.test(s);
    },
    stringContainsSpace: function (s) {
        return (/\s/).test(s);
    },
    validatorPattern: function (reg, s) {
        return reg.test(s);
    },
    //输入框输入验证
    inputs: function (els) {
        els.each(function (index, el) {
            if ($(this).attr('number-width')) {
                this.lastValue = this.value;
                this.dataWidth = $(this).attr('number-width') - 0;
            }
        });
        els.on('input', function () {
            var value = this.value;
            if ($(this).attr('number-width')) {
                if (!value && this.lastValue.length === 1) {
                    this.value = '';
                    this.lastValue = '';
                    return;
                }
                if (!value || isNaN(value) || value.length > this.dataWidth || value - 0 < 0) {
                    this.value = this.lastValue;
                    return;
                }
                this.lastValue = value;
            }
        });
        els.on('focus', function () {
            if (this.value)
                $(this).next('.empty-input').removeClass('ui-hidden'); else
                $(this).next('.empty-input').addClass('ui-hidden');
        });
        els.on('blur', function (event) {
            $(this).next('.empty-input').addClass('ui-hidden');
        });
        els.on('input', function () {
            if (this.value)
                $(this).next('.empty-input').removeClass('ui-hidden'); else
                $(this).next('.empty-input').addClass('ui-hidden');
        });
    }
};

// ------------------------------------------------------------------------
// Motion track extension for jQuery
//
(function ($) {
    'use strict';

    var MOTION_TYPE_H = 0, MOTION_TYPE_V = 1, MOTION_TYPE_HV = 2, TRACK_XY_THRESHOLD = 8, // in px
        hasTouch = 'ontouchstart' in window, TOUCH_EVENT = hasTouch ? 'touchstart.__mt touchend.__mt touchcancel.__mt touchmove.__mt' : 'mousedown.__mt mouseup.__mt mousemove.__mt', defaultDelegate = {
            onTrackStart: function (target, pt) {
            },
            onTrackMove: function (target, pt) {
            },
            onTrackEnd: function (target) {
            }
        };

    $.fn.motionTrack = function (op, opts) {
        return this.each(function () {
            var el = $(this);
            if (op === 'start') {
                if (!opts.selector || typeof opts.selector !== 'string')
                    return;
                startMotionTrack(el, getMoveType(opts.type), opts.selector, $.extend({}, defaultDelegate, opts.delegate));
            } else if (op === 'stop') {
                stopMotionTrack(el);
            }
        });
    };

    function getMoveType(str) {
        if (str === 'h')
            return MOTION_TYPE_H; else if (str === 'v')
            return MOTION_TYPE_V; else
            return MOTION_TYPE_HV;
    }

    function shouldStartTrack(motionType, dx, dy) {
        dx = Math.abs(dx);
        dy = Math.abs(dy);
        switch (motionType) {
            case MOTION_TYPE_H:
                return dx > TRACK_XY_THRESHOLD && dx > dy;
            case MOTION_TYPE_V:
                return dy > TRACK_XY_THRESHOLD && dy > dx;
            //case MOTION_TYPE_HV:
            default:
                return Math.abs(dx) > TRACK_XY_THRESHOLD || Math.abs(dy) > TRACK_XY_THRESHOLD;
        }
    }

    function startMotionTrack(el, motionType, selector, delegate) {
        var trackInitialized = false, trackStarted = false, originPt = null, target = null;

        el.on(TOUCH_EVENT, selector, function (e) {
            var _t = e.type, ev;

            if (_t === 'touchstart' || _t === 'mousedown') {
                if (target && target !== this && trackStarted) {
                    delegate.onTrackEnd(target);
                    trackStarted = false;
                }
                ev = hasTouch ? e.originalEvent.touches[0] : e;
                originPt = { x: ev.pageX, y: ev.pageY };
                trackInitialized = true;
                target = this;
            } else if (target === this && (_t === 'touchmove' || _t === 'mousemove')) {
                if (!trackInitialized)
                    return;
                ev = hasTouch ? e.originalEvent.touches[0] : e;
                if (!trackStarted && shouldStartTrack(motionType, ev.pageX - originPt.x, ev.pageY - originPt.y)) {
                    trackStarted = true;
                    delegate.onTrackStart(target, originPt);
                    Util.inMotionTrack = true;
                }
                if (trackStarted) {
                    delegate.onTrackMove(target, { x: ev.pageX, y: ev.pageY });
                    e.stopPropagation();
                }
            } else if (trackInitialized) { // _t === 'touchend' || _t === 'touchcancel' || _t === 'mouseup' || _t === 'mouseout'
                if (trackStarted) {
                    delegate.onTrackEnd(target);
                    //e.stopPropagation();
                    setTimeout(function () {
                        Util.inMotionTrack = false;
                    }, 50);
                }
                trackInitialized = false;
                trackStarted = false;
                originPt = null;
                target = null;
            }
        });
    }

    function stopMotionTrack(el) {
        el.off(TOUCH_EVENT);
    }
})(jQuery);

// ------------------------------------------------------------------------
// DateUtil
//
var DateUtil = {
    ONE_DAY_MILLSECONDS: 86400000,
    ONE_WEEK_MILLSECONDS: 604800000,
    HALF_MONTH_MILLSECONDS: 1296000000,
    pad2digits: function (n) {
        return n < 10 ? '0' + n : '' + n;
    },
    formatDate: function (date) {
        return date.getUTCFullYear() + '-' + this.pad2digits(date.getUTCMonth() + 1) + '-' + this.pad2digits(date.getUTCDate()) + ' ' + this.pad2digits(date.getHours()) + ':' + this.pad2digits(date.getMinutes());
    },
    formatYY_MM_DD: function (date) {
        return this.formatWithPatternDate('yyyy-mm-dd', date);
    },
    formatYYMMDD: function (date) {
        return this.formatWithPatternDate('yyyymmdd', date);
    },
    formatWithPatternDate: function (format, date) {
        if (!date) {
            return null;
        }
        var look = function (m) { // Check whether a format character is doubled
            var n = 0;
            while (i + 1 < format.length && format.charAt(i + 1) === m) {
                n++;
                i++;
            }
            return n;
        }, f1 = function (m, val, len) { // Format a number, with leading zero if necessary
            var n = '' + val;
            if (look(m)) {
                while (n.length < len) {
                    n = '0' + n;
                }
            }
            return n;
        }, i, output = '', literal = false;

        for (i = 0; i < format.length; i++) {
            if (literal) {
                if (format.charAt(i) === "'" && !look("'")) {
                    literal = false;
                } else {
                    output += format.charAt(i);
                }
            } else {
                switch (format.charAt(i)) {
                    case 'd':
                        output += f1('d', date.getDate(), 2);
                        break;
                    case 'o':
                        output += f1('o', (date.getTime() - new Date(date.getFullYear(), 0, 0).getTime()) / 86400000, 3);
                        break;
                    case 'm':
                        output += f1('m', date.getMonth() + 1, 2);
                        break;
                    case 'y':
                        output += (look('y') ? date.getFullYear() : (date.getYear() % 100 < 10 ? '0' : '') + date.getYear() % 100);
                        break;
                    case 'h':
                        var h = date.getHours();
                        output += f1('h', (h > 12 ? (h - 12) : (h === 0 ? 12 : h)), 2);
                        break;
                    case 'H':
                        output += f1('H', date.getHours(), 2);
                        break;
                    case 'i':
                        output += f1('i', date.getMinutes(), 2);
                        break;
                    case 's':
                        output += f1('s', date.getSeconds(), 2);
                        break;
                    case 'a':
                        output += date.getHours() > 11 ? '下午' : '上午';
                        break;
                    case 'A':
                        output += date.getHours() > 11 ? 'PM' : 'AM';
                        break;
                    case "'":
                        if (look("'")) {
                            output += "'";
                        } else {
                            literal = true;
                        }
                        break;
                    default:
                        output += format.charAt(i);
                }
            }
        }
        return output;
    },
	getSpaceTime: function(spaceTime) {
		var minuteTime = 1000*60;
		var hourTime = 60*minuteTime;
		var dateTime = 24*hourTime ;//一天的毫秒数
		var minute = spaceTime/1000;
		var space = "";
		if(spaceTime<minuteTime){
			space = "刚刚";
		}else if(spaceTime<hourTime){
			space = Math.floor(minute/60)+"分钟前";
		}else if(spaceTime<dateTime){
			space = Math.floor(minute/60/60)+"小时前";
		}else if(spaceTime<7*dateTime){
			space = Math.floor(minute/60/60/24)+"天前";
		}else{
			space = this.formatYY_MM_DD(new Date(new Date().getTime()-spaceTime));
		}
		return space;
	},

	getFutureSpaceTime: function(spaceTime) {
		var c = spaceTime/1000;
		var d,h,m,s;
		d=Math.floor(c/24/3600);
		h=Math.floor((c-d*24*3600)/3600);
		m=Math.floor((c-d*24*3600-h*3600)/60);
		s=Math.floor(c-d*24*3600-h*3600-m*60);

		var resultStr;
		if(d>0){
			resultStr = d+'天'+h+'小时'+m+'分';
		}
		else if(h>0){
			resultStr = h+'小时'+m+'分';
		}
		else if(m>0){
			resultStr = m+'分'+s+'秒';
		}
		else if(s>0){
			resultStr = s+'秒';
		}

		return  resultStr;
	},

	getFutureSpaceTime2: function(spaceTime) {
		var c = spaceTime/1000;
		var d,h,m,s;
		d=Math.floor(c/24/3600);
		h=Math.floor((c-d*24*3600)/3600);
		m=Math.floor((c-d*24*3600-h*3600)/60);
		s=Math.floor(c-d*24*3600-h*3600-m*60);

		var resultStr;
		if(d>0){
			resultStr = d+'天';
		}
		else if(h>0){
			resultStr = h+'小时';
		}
		else if(m>0){
			resultStr = m+'分钟';
		}
		else{
			resultStr = s+'秒';
		}

		return  resultStr;
	},

	//获取电话咨询的截止时间
	getEndDay: function (spaceTime) {
		var c = spaceTime/1000;
		var d,h,m,s;
		d=Math.floor(c/24/3600);
		h=Math.floor((c-d*24*3600)/3600);
		m=Math.floor((c-d*24*3600-h*3600)/60);
		s=Math.floor(c-d*24*3600-h*3600-m*60);

		if(d>0){
			return d;
		}
		else{
			return 1;
		}
	}
};

// ------------------------------------------------------------------------
// String and Array utilities
//
(function () {
    'use strict';

    if (typeof String.prototype.startsWith === 'undefined') {
        String.prototype.startsWith = function (s) {
            if (!s) return false;
            return this.indexOf(s) === 0;
        };
    }

    if (typeof String.prototype.endsWith === 'undefined') {
        String.prototype.endsWith = function (s) {
            if (!s || !s.length) return false;
            return this.lastIndexOf(s) === this.length - s.length;
        };
    }

    if (typeof Array.prototype.insert === 'undefined') {
        Array.prototype.insert = function (index, item) {
            this.splice(index, 0, item);
            return true;
        };
    }

    if (typeof Array.prototype.remove === 'undefined') {
        Array.prototype.remove = function (index) {
            var arr = this.splice(index, 1);
            return (arr && arr.length === 1) ? arr[0] : null;
        };
    }

    if (typeof Array.prototype.first === 'undefined') {
        Array.prototype.first = function () {
            return this.length > 0 ? this[0] : null;
        };
    }

    if (typeof Array.prototype.last === 'undefined') {
        Array.prototype.last = function () {
            return this.length > 0 ? this[this.length - 1] : null;
        };
    }
	if(typeof String.prototype.replaceAll === 'undefined') {
		String.prototype.replaceAll = function(s1, s2) {
			return this.replace(new RegExp(s1, "gm"), s2);
		}
	}
})();

/*
 * used for local save
 * model : maternal or baby and so on
 * */
var LocalMode = function (model) {
    var o = null;
    (function () {
        if (Util.data(model) === undefined) {
            Util.data(model, {});
        }
    })();
    o = {
        model: model,
        get: function () {
            return Util.data(this.model);
        },
        update: function (obj) {
            var oldData = this.get(), newData = $.extend(oldData, obj, {code: UserProfile.code});
            Util.data(this.model, newData);
            return this;
        },
        remove: function () {
            Util.data(this.model, {});
        }
    };
    return o;
};

/**
 * Function.prototype.bind
 * */
(function () {
    if (!Function.prototype.bind) {
        Function.prototype.bind = function (oThis) {
            if (typeof this !== "function") {
                // closest thing possible to the ECMAScript 5 internal IsCallable function
                throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
            }

            var aArgs = Array.prototype.slice.call(arguments, 1), fToBind = this, fNOP = function () {
            }, fBound = function () {
                return fToBind.apply(this instanceof fNOP && oThis ? this : oThis || window, aArgs.concat(Array.prototype.slice.call(arguments)));
            };

            fNOP.prototype = this.prototype;
            fBound.prototype = new fNOP();

            return fBound();
        };
    }
})();

var VoiceUtil = {
	startRecord: function () {
		var onSuccess = function (winParam) {
		};
		var onFail = function () {
			alert('fail');
		};
		if(mobile_system == 'android'){
			cordova.exec(onSuccess, onFail, "recordAction", "startRecord", '');
		}
		else{
			var fileName = Util.rtime();
			var options = {
				isRecord: true,
				src: fileName.toString(),
				recordSuccess: function () {
//                    alert('1216: record success');
				},
				recordError: function () {
//                    alert('1219: record error');
				}
			};
			Views.consultDetailView.localVoicePath = options.src+'.ilbc';
			Resource.media.init(options);
			Resource.media.start();
		}
	},

	stopRecord: function () {
		if(mobile_system == 'android'){
			var view = Views.consultDetailView;
			var onSuccess = function (path) {
				var opt1 = {
					file: path,
					url: WEB_URL_R + "/audio",
					data: {
						app: 'soft-consult'
					},
					onSuccess: 'voiceUploadedSuccess',
					onError: 'voiceUploadedFail'
				};
				Widget.popupLoading('语音发送中…');
				view.localVoicePath = path;
				FSManager.upload(opt1);
			};
			var onFail = function () {
				Widget.popupToast('语音录制失败');
			};
			cordova.exec(onSuccess, onFail, "recordAction", "stopRecord", '');
		}
		else{
			var view = Views.consultDetailView;
			if(view.lastLocalVoicePath !== view.localVoicePath){
				Resource.media.stop();
				var opt = {
					file: FSManager.getRootPath() + '/' + view.localVoicePath,
					url: WEB_URL_R + "/audio",
					data: {
						app: 'soft-consult'
					},
					onSuccess: 'voiceUploadedSuccess',
					onError: 'voiceUploadedFail'
				};
				view.lastLocalVoicePath = view.localVoicePath;
				Widget.popupLoading('语音发送中…');
				FSManager.upload(opt);
			}
			else{
				$('.voice-board').addClass('ui-btn');//在upload时去掉的要加回去
			}
		}
	},

	playRecord: function (path, pattern) {
		if(pattern == 'android'){
			var onSuccess = function (winParam) {
			};
			var onFail = function () {
			};
			// android本地录制的音频播放
			cordova.exec(onSuccess, onFail, "recordAction", "mediaPlay", path);
		}
		else if(pattern == 'cordova'){
			// android下载，ios本地录制及下载 的播放方式
			var options = {
				src: path,
				playSuccess: function () {
//                alert('成功');
				},
				playError: function () {
//                alert('失败');
				}
			}
			Resource.media.init(options);
			Resource.media.start();
		}
	},

	stopPlay: function () {
		var onSuccess = function (winParam) {
			Widget.popupToast(path);
		};
		var onFail = function () {
		};
		cordova.exec(onSuccess, onFail, "recordAction", "stopPlay", '');
	}
};

var Resource = {
	media: {
		inst: null,
		isRecord: false,
		getRootPath: function () {
			return FSManager.getRootPath();
		},
		clear: function () {
			if (this.inst) {
				this.inst.release();
				this.inst = null;
			}
		},

		// options.isRecord(boolean) 默认是播放
		init: function (options) {
			//初始化之前清空上次数据
			this.clear();
			if(mobile_system === 'ios'){
				// ios不需要额外的文件夹目录
				options.src = options.src.substr(options.src.lastIndexOf('/')+1);
			}
			function getFullPath (src) {
				if(mobile_system === 'ios'){
					return FSManager.getRootPath() + "/" + src + ".ilbc";
				}
			}

			this.isRecord = options.isRecord || false;

			if (this.isRecord) {
				this.inst = new Media(getFullPath(options.src),
					options.recordSuccess, options.recordError);
			} else {
				this.inst = new Media(options.src, options.playSuccess, options.playError);
			}


		},
		//开始
		start: function () {
			if (this.inst) {
				if (this.isRecord) {
					this.inst.startRecord();
				} else {
					this.inst.play();
				}
			}
		},
		//结束
		stop: function () {
			if (this.inst) {
				if (this.isRecord) {
					this.inst.stopRecord();
					return this.inst.getDuration();
				} else {
					this.inst.stop();
				}
			}
		},
		//options.src 文件名
		upload: function (options) {
			var uploadOptions = new FileUploadOptions();
			uploadOptions.fileKey  = 'file';
			uploadOptions.fileName = options.src;
			uploadOptions.params   = {
				"file": options.src,
				"app": "mobileclinic"
			};
			var ft = new FileTransfer();
//			function uploadSuccess(resp){
//				//上传ID uploadID 可以从这种方法获得
//				Resource.media.uploadID = JSON.parse(resp.response).data;
//				console.log(arguments);
//			}

//			function uploadError(){
//				console.log(4);
//				console.log(arguments);
//			}
			ft.upload(this.localURL, encodeURI( WEB_URL_R + "/audio"),
				options.uploadSuccess ,options.uploadError, uploadOptions);
		},
		//下载ilbc
		download: function (options) {
			var ft = new FileTransfer();
			ft.download(encodeURI(WEB_URL_R + "/audio/ilbc/" + options.src),
				FSManager.getRootPath() + "/" + options.src, options.downloadSuccess, options.downloadError);

		}
	},
	image: {

	}
};